using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Net;
using System.Windows.Forms;

namespace FSDSD_Client
{
    public class FSDSD_Game : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        List<SpriteFont> FontList;
        List<Texture2D> TextureList;

        FSDSD_Server_Interface serverinterface;
        int PlayerNumber, DestinationType, DestinationIndex, lasthitpoints;
        Universe UniverseVar;
        float MouseX, MouseY,playerposx, playerposy, stationrotation;
        InputManager Input;
        Vector2 PlayerScreenPosition, Upvector;

        RenderTarget2D FieldmapRenderTarget, MinimapRenderTarget, InfoscreenRenderTarget;
        Texture2D FieldmapTexture, MinimapTexture, InfoscreenTexture;
        DepthStencilBuffer oldBuffer;
        List<Vector2> Starfield;
        Random RandomNumberGenerator;
        bool ShowMessages;
//        StreamWriter outfile;

        AudioEngine audio;
        WaveBank waves;
        SoundBank sounds;

        public FSDSD_Game()
        {
            
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            graphics.PreferredBackBufferWidth = 1024;
            graphics.PreferredBackBufferHeight = 768;
            graphics.IsFullScreen = false;
            graphics.PreferredDepthStencilFormat = DepthFormat.Depth32;
            graphics.PreferMultiSampling = true;
            graphics.SynchronizeWithVerticalRetrace = true;

            PlayerScreenPosition = new Vector2();
        }

        protected override void Initialize()
        {// Allows the game to perform any initialization it needs to before starting to run. This is where it can query for any required services and load any non-graphic related content.  Calling base.Initialize will enumerate through any components and initialize them as well.
            try
            {
                serverinterface = new FSDSD_Server_Interface();
                PlayerNumber = serverinterface.RegisterClientwithServer();  // player number ist die id mit der sich jeder spieler bei aktionen identifiziert
                if (PlayerNumber < 0)
                {// maximale Teilnehmerzahl erreicht also melden und beenden
                    //# messagebox o "No more Clients allowed. Adjust GameSettings.ini at the Server and restart the Server."
                    Exit();
                }
                audio = new AudioEngine("Content\\Sounds.xgs");
                waves = new WaveBank(audio, "Content\\Wave Bank.xwb");
                sounds = new SoundBank(audio, "Content\\Sound Bank.xsb");
            }
            catch (WebException)
            {
                //# messagebox, abfragen ob server nicht gefunden (anderer grund?) server nicht gestartet oder netzwerkkonfig falsch (firewall),
                Exit();
            }
//            outfile = new StreamWriter("C:\\Downloads\\testoutput.txt"); // Datei ffnen
            FontList = new List<SpriteFont>();
            TextureList = new List<Texture2D>();
            spriteBatch = new SpriteBatch(GraphicsDevice);// Create a new SpriteBatch, which can be used to draw textures.
            Input = new InputManager();
            Input.Init();
            PlayerScreenPosition.X = (float)(GraphicsDevice.PresentationParameters.BackBufferHeight / 2);   // die height statt width hier ist beabsichtigt damit der spieler im linken quadratischen teil zentriert wird (rechts ist minimap + infos)
            PlayerScreenPosition.Y = (float)(GraphicsDevice.PresentationParameters.BackBufferHeight / 2);
            Upvector = new Vector2(0.0f, 1.0f);
            Starfield = new List<Vector2>();
            ShowMessages = true;
            DestinationType = 0;    // starten mit nahester station/frachter fr zielanzeige
            DestinationIndex = -1;
            lasthitpoints = 1000;
            RandomNumberGenerator = new Random();
            for (int i = 0; i < 30; i++)    // anzahl stars anpassen
                Starfield.Add(new Vector2((float)RandomNumberGenerator.Next(768), (float)RandomNumberGenerator.Next(768)));
            base.Initialize();
        }

        protected override void LoadContent()
        {// LoadContent will be called once per game and is the place to load all of your content.
            FontList.Add(Content.Load<SpriteFont>("Crystal10"));    // #0
            FontList.Add(Content.Load<SpriteFont>("Crystal12"));    // #1
            FontList.Add(Content.Load<SpriteFont>("Crystal13"));    // #2
            FontList.Add(Content.Load<SpriteFont>("Crystal14"));    // #3
            FontList.Add(Content.Load<SpriteFont>("Crystal16"));    // #4
            FontList.Add(Content.Load<SpriteFont>("Crystal18"));    // #5
            FontList.Add(Content.Load<SpriteFont>("Crystal20"));    // #6
            FontList.Add(Content.Load<SpriteFont>("Crystal22"));    // #7
            FontList.Add(Content.Load<SpriteFont>("Crystal26"));    // #8
            TextureList.Add(Content.Load<Texture2D>("MouseCursorImage"));  // #0
            TextureList.Add(Content.Load<Texture2D>("Asteroid_01"));  // #1
            TextureList.Add(Content.Load<Texture2D>("Asteroid_02"));  // #2
            TextureList.Add(Content.Load<Texture2D>("Asteroid_03"));  // #3
            TextureList.Add(Content.Load<Texture2D>("Asteroid_04"));  // #4
            TextureList.Add(Content.Load<Texture2D>("Asteroid_05"));  // #5
            TextureList.Add(Content.Load<Texture2D>("Asteroid_Station"));  // #6
            TextureList.Add(Content.Load<Texture2D>("Frachter"));  // #7
            TextureList.Add(Content.Load<Texture2D>("Raumschiff"));  // #8
            TextureList.Add(Content.Load<Texture2D>("Raumschiff_firing"));  // #9
            TextureList.Add(Content.Load<Texture2D>("DockingKreis"));  // #10
            TextureList.Add(Content.Load<Texture2D>("MMAsteroid"));  // #11
            TextureList.Add(Content.Load<Texture2D>("MMForeignShip"));  // #12
            TextureList.Add(Content.Load<Texture2D>("MMFreighter"));  // #13
            TextureList.Add(Content.Load<Texture2D>("MMOwnShip"));  // #14
            TextureList.Add(Content.Load<Texture2D>("MMStation"));  // #15
            TextureList.Add(Content.Load<Texture2D>("ArrowShip"));  // #16
            TextureList.Add(Content.Load<Texture2D>("ArrowStation"));  // #17
            TextureList.Add(Content.Load<Texture2D>("MMScreenRect"));  // #18
            FieldmapRenderTarget = new RenderTarget2D(graphics.GraphicsDevice, 768, 768, 1, graphics.GraphicsDevice.DisplayMode.Format);
            MinimapRenderTarget = new RenderTarget2D(graphics.GraphicsDevice, 256, 256, 1, graphics.GraphicsDevice.DisplayMode.Format);
            InfoscreenRenderTarget = new RenderTarget2D(graphics.GraphicsDevice, 256, 512, 1, graphics.GraphicsDevice.DisplayMode.Format);
        }

        protected override void UnloadContent()
        {// UnloadContent will be called once per game and is the place to unload all content.
//            outfile.Close();
        }

        protected override void Update(GameTime gameTime)
        {// Allows the game to run logic such as updating the world, checking for collisions, gathering input, and playing audio.
            float secondfraction = (float)gameTime.ElapsedGameTime.Milliseconds / 1000.0f;   // wird an server bergeben um getimte sachen zu bestimmen (zb bewegung seit letztem frame)
            try
            {
                if (1 == serverinterface.CheckUpdateState(PlayerNumber))
                {// es ist gengend zeit seit dem letzten abfragen verstrichen also alle daten beim server holen
/*                    try
                    {
                        outfile.Write(String.Format("client x:{0} y:{1} dx:{2} dy:{3}\n", UniverseVar.Elements[30].Position_X, UniverseVar.Elements[30].Position_Y, UniverseVar.Elements[30].Velocity_X, UniverseVar.Elements[30].Velocity_Y));
                    }
                    catch{};*/
                    do
                    {
                        UniverseVar = serverinterface.GetUniverse();  // hier alle daten abholen, falls locked wegen kopieren wird 0 zurckgegeben so lange weiterversuchen
                    } while (null == UniverseVar);
//                    outfile.Write(String.Format("server x:{0} y:{1} dx:{2} dy:{3}\n", UniverseVar.Elements[30].Position_X, UniverseVar.Elements[30].Position_Y, UniverseVar.Elements[30].Velocity_X, UniverseVar.Elements[30].Velocity_Y));
                    if (((Player)(UniverseVar.Elements[PlayerNumber])).ActHitpoints < lasthitpoints)
                        sounds.PlayCue("Collision_mod");
                    lasthitpoints = ((Player)(UniverseVar.Elements[PlayerNumber])).ActHitpoints;
                }
                else
                {// alle elemente mit letzten infos bewegen (kommt nicht vom server) um sprnge zu verhindern bei geringen abfrageraten, falls kollision eintritt wird die nicht bercksichtigt und erst beim nchsten update vom server werden die neuen vektoren bernommen was zu sprngen zwischen zwei frames fhren kann
                    if (false == UniverseVar.GamePaused) // nur bewegen wenn nicht pausiert
                        MoveFieldElements(secondfraction);
                }
            }
            catch (WebException)
            {
                //# Meldung ausgeben "Kontakt zum Server verloren. Bitte alles neu starten."
            }

            // input prfen und drauf reagieren
            Input.Update(gameTime);
            MouseX = Input.GetMousePosX(); // update der koordinaten fr die maus
            MouseY = Input.GetMousePosY();
            if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.Escape))   // bei esc server benachrichtigen und spiel verlassen
            {
                serverinterface.ClientTerminates(PlayerNumber);
                Exit();
            }
            if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.Pause))   // pause taste pausiert spiel im singleplayer
                serverinterface.PauseGame(PlayerNumber);
            if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.F12)) // f12 schaltet zwischen vollbild und fenstermodus um
                graphics.ToggleFullScreen();
            if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.M))   // messages (de)aktivieren
                ShowMessages = !ShowMessages;
            if ((false == ((Player)(UniverseVar.Elements[PlayerNumber])).Lost) && (false == UniverseVar.GamePaused))
            {// nur wenn spieler noch nicht verloren hat und spiel nicht pausiert ist input entgegen nehmen und verarbeiten
                serverinterface.UpdateRotation(PlayerNumber, PlayerScreenPosition.X - MouseX, PlayerScreenPosition.Y - MouseY);   // dem server mitteilen wo spieler hinrotieren mchte (rotation wird auf server berechnet)
                if ((Input.IsPressed(0)) || (Input.IsPressed(Microsoft.Xna.Framework.Input.Keys.LeftControl))) // linke maustaste oder links ctrl aktiviert triebwerke in aktuelle richtung (nicht dahin wo sich maus befindet!)
                {
                    serverinterface.FireThrusters(PlayerNumber);
                    if(-1 == ((Player)(UniverseVar.Elements[PlayerNumber])).Dockingindex)   // nur wenn nicht gedockt sound spielen
                        sounds.PlayCue("Thrusters_mod");
                }
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.Space))
                    serverinterface.Brake(PlayerNumber);
                if ((Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.D)) || (Input.IsFirstPressed(1)))
                    serverinterface.Docking(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.T))   // wo ich gerade bin handeln
                    serverinterface.Trade(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.R))   // panzerung reparieren
                    serverinterface.Repair(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.F))   // treibstofftanks fllen (tanken)
                    serverinterface.Refuel(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.L))   // cargoupgrade kaufen (laderaum)
                    serverinterface.BuyCargoUpgrade(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.K))   // treibstoffupgrade kaufen (tankkapazitt)
                    serverinterface.BuyFuelUpgrade(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.P))   // panzerungsupgrade kaufen (panzerung, hitpoints)
                    serverinterface.BuyHitpointUpgrade(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.A))   // wendigkeitsupgrade kaufen (agility)
                    serverinterface.BuyAgilityUpgrade(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.S))   // triebwerksupgrade kaufen (thrusters)
                    serverinterface.BuyAccelerationUpgrade(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.B))   // bremssupgrade kaufen (brake)
                    serverinterface.BuyBrakeUpgrade(PlayerNumber);
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.N))
                    NextDestination();  // die ziele bei jedem drcken in funktion durchiterieren
                if (Input.IsFirstPressed(Microsoft.Xna.Framework.Input.Keys.E))
                    DestinationType = 0;    // das naheste Ziel anzeigen
            }
            audio.Update();
            base.Update(gameTime);
        }
        protected void MoveFieldElements(float secondfraction)
        {// alle elemente werden entsprechend ihres bewegungsvektors bewegt, diese funktion wurde vom server kopiert um die positionen lokal zu extrapolieren um bei geringer abfragerate flssige bewegungen zu ermglichen, die daten werden regelmig vom server berschrieben und falls die anders sind (zb nach kollision die nicht lokal berechnet wird) kann es zu sprngen kommen
            if (UniverseVar.GamePaused)
                return; // wenn game pausiert ist nichts machen
            for (int i = 0; i < UniverseVar.Elements.Count; i++)
            {
                UniverseVar.Elements[i].Position_X += UniverseVar.Elements[i].Velocity_X * secondfraction;
                UniverseVar.Elements[i].Position_Y += UniverseVar.Elements[i].Velocity_Y * secondfraction;
                if (UniverseVar.Elements[i].Type < 7)   // stationen und asteroiden rotieren, frachter rotieren nicht und schiffe werden durch steuerung rotiert,
                    UniverseVar.Elements[i].RotationAngle += UniverseVar.Elements[i].RotationSpeed * secondfraction;
                if ((8 == UniverseVar.Elements[i].Type) && (((Player)(UniverseVar.Elements[i])).Dockingindex >= 0)) // das schiff ist wo gedockt also die schiffsposition auf die stations-/frachterposition beziehene
                {
                    UniverseVar.Elements[i].Position_X = UniverseVar.Elements[((Player)(UniverseVar.Elements[i])).Dockingindex].Position_X + ((Player)(UniverseVar.Elements[i])).DockedPos_X;
                    UniverseVar.Elements[i].Position_Y = UniverseVar.Elements[((Player)(UniverseVar.Elements[i])).Dockingindex].Position_Y + ((Player)(UniverseVar.Elements[i])).DockedPos_Y;
                }
            }//# warum wird das hier fr spieler auch gemacht und dann unten nochmal?
            //# der spieler wird auch vor station bewegt also wenn station spter angepasst wird ist auch wieder sprung drin
            float pi = (float)Math.PI, targetrotation = GetRotation(new Vector2(MouseX - PlayerScreenPosition.X, MouseY - PlayerScreenPosition.Y) * -1.0f);
            Player player = (Player)UniverseVar.Elements[PlayerNumber];
            if (player.RotationAngle < 0.0f)
            {// schiff zeigt nach links
                if ((player.RotationAngle + pi > targetrotation) && (player.RotationAngle < targetrotation)) // nach rechts drehen
                    player.RotationAngle += player.ActAgility() * secondfraction;
                else
                    player.RotationAngle -= player.ActAgility() * secondfraction;
            }
            else
            {// schiff zeigt nach rechts
                if ((player.RotationAngle - pi < targetrotation) && (player.RotationAngle > targetrotation)) // nach links drehen
                    player.RotationAngle -= player.ActAgility() * secondfraction;
                else
                    player.RotationAngle += player.ActAgility() * secondfraction;
            }
            if (player.RotationAngle > pi)  // zu groe und zu kleine werte anpassen (-pi < winkel < pi)
                player.RotationAngle -= pi * 2.0f;
            if (player.RotationAngle < pi * -1.0f)
                player.RotationAngle += pi * 2.0f;
        }
        protected void NextDestination()
        {// schaltet ziele durch (stationen + frachter)
            if (0 == DestinationType)   // der andere modus war vorher aktiv also bei 0 beginnen
            {
                DestinationIndex = -1;
                DestinationType = 1;
            }
            do
            {
                if (++DestinationIndex >= UniverseVar.Elements.Count)
                    DestinationIndex = 0;
            } while ((6 != UniverseVar.Elements[DestinationIndex].Type) && (7 != UniverseVar.Elements[DestinationIndex].Type)); // so lange iterieren bis eine station oder frachter gefunden wird
        }
        protected override void Draw(GameTime gameTime)
        {// This is called when the game should draw itself
            GraphicsDevice.Clear(Color.Pink);
            RenderFieldmap((float)gameTime.ElapsedGameTime.Milliseconds * 0.0001f);   // spielfeld in textur rendern
            RenderMinimap();   // minimap in textur rendern
            RenderInfoscreen();   // infos in textur rendern
            spriteBatch.Begin();
            spriteBatch.Draw(FieldmapTexture, new Vector2(0.0f, 0.0f), Color.White);    // spielfeld aus textur auf bildschirm rendern
            spriteBatch.Draw(MinimapTexture, new Vector2(768.0f, 0.0f), Color.White);    // minimap aus textur auf bildschirm rendern
            spriteBatch.Draw(InfoscreenTexture, new Vector2(768.0f, 256.0f), Color.White);    // infos aus textur auf bildschirm rendern
            DrawImage(new Vector2(MouseX - 12f, MouseY - 12f), 0, 0.0f);    // mauszeiger zuletzt zeichnen
            spriteBatch.End();
            base.Draw(gameTime);
        }

        protected void DrawImage(Vector2 position, int textureindex, float rotation)
        {// zeichnet eine textur aus der liste an die gewhlte position
            spriteBatch.Draw(TextureList[textureindex], position, null, Color.White,rotation,new Vector2(TextureList[textureindex].Height/2,TextureList[textureindex].Height/2),1.0f,SpriteEffects.None,0.0f);
        }
        private int Transform(out Vector2 ObjScreenPosition, int arraypos)
        {// transformiert die objektkoordinaten (welt) in screenkoordinaten frs rendern
            ObjScreenPosition.X = PlayerScreenPosition.X + (float)(UniverseVar.Elements[arraypos].Position_X - playerposx);
            ObjScreenPosition.Y = PlayerScreenPosition.Y + (float)(UniverseVar.Elements[arraypos].Position_Y - playerposy);
            return 1;   //# hier noch sichtbarkeitsprfung machen
        }
        protected void RenderFieldmap(float secondfraction)
        {// rendert das spielfeld in eine textur
            oldBuffer = graphics.GraphicsDevice.DepthStencilBuffer;
            GraphicsDevice.SetRenderTarget(0,FieldmapRenderTarget); // rendertarget setzen
            graphics.GraphicsDevice.DepthStencilBuffer = CreateDepthStencil(FieldmapRenderTarget);  // angepassten ds-buffer setzen
            spriteBatch.Begin();
            GraphicsDevice.Clear(Color.Black);
            Vector2 ObjScreenPosition, ObjectVector;
            String namestring = "" ;
            int minindex=0,i;
            float mindistance = float.MaxValue;
            playerposx = UniverseVar.Elements[PlayerNumber].Position_X;
            playerposy = UniverseVar.Elements[PlayerNumber].Position_Y;
            for (i = 0; i < 30; i++)    // anzahl stars anpassen falls gewnscht
            {// das starfield zeichnen und elemente bewegen
                Vector2 temp1 = Starfield[i];
                if (false == UniverseVar.GamePaused)
                {// wenn pausiert nicht bewegen (aber zeichnen)
                    if (-1 == ((Player)(UniverseVar.Elements[PlayerNumber])).Dockingindex)
                    {// normaler modus also sterne in gegenrichtung zum spieler bewegen
                        temp1.X -= UniverseVar.Elements[PlayerNumber].Velocity_X * secondfraction * 10.0f;
                        temp1.Y -= UniverseVar.Elements[PlayerNumber].Velocity_Y * secondfraction * 10.0f;
                    }
                    else
                    {// gedockter modus also sterne in gegenrichtung zur station/frachter bewegen
                        temp1.X -= UniverseVar.Elements[((Player)(UniverseVar.Elements[PlayerNumber])).Dockingindex].Velocity_X * secondfraction * 10.0f;
                        temp1.Y -= UniverseVar.Elements[((Player)(UniverseVar.Elements[PlayerNumber])).Dockingindex].Velocity_Y * secondfraction * 10.0f;
                    }
                    Starfield[i] = temp1;
                    if (Starfield[i].X < 0.0f)
                    {
                        Vector2 temp = new Vector2(768.0f, (float)RandomNumberGenerator.Next(768));
                        Starfield[i] = temp;
                    }
                    if (Starfield[i].X > 768.0f)
                    {
                        Vector2 temp = new Vector2(0.0f, (float)RandomNumberGenerator.Next(768));
                        Starfield[i] = temp;
                    }
                    if (Starfield[i].Y < 0.0f)
                    {
                        Vector2 temp = new Vector2((float)RandomNumberGenerator.Next(768), 768.0f);
                        Starfield[i] = temp;
                    }
                    if (Starfield[i].Y > 768.0f)
                    {
                        Vector2 temp = new Vector2((float)RandomNumberGenerator.Next(768), 0.0f);
                        Starfield[i] = temp;
                    }
                }
                spriteBatch.Draw(TextureList[11], Starfield[i], null, Color.White, 0.0f, new Vector2(TextureList[11].Height / 2, TextureList[11].Height / 2), 0.5f, SpriteEffects.None, 0.0f);  // den stern zeichnen (bild von asteroid)
            }
            for (i = 0; i < UniverseVar.Elements.Count; i++)
            {// alle elemente zeichnen
                if(0 == Transform(out ObjScreenPosition,i) )   // transformation der welt-koordinaten in screen koordinaten, wenn 0 nicht rendern da nicht sichtbar
                    continue;
                if(UniverseVar.Elements[i].Dockable)    // bei dockbaren auch den kreis drunterrendern
                    DrawImage(ObjScreenPosition, 10, 0.0f);
                if ((8 == UniverseVar.Elements[i].Type) && (true == ((Player)UniverseVar.Elements[i]).ThrustersFiring))   // wenns ein schiff/Player ist und die triebwerke an sind entsprechendes bild zeichnen
                    DrawImage(ObjScreenPosition, 9, UniverseVar.Elements[i].RotationAngle);
                else
                {
                    DrawImage(ObjScreenPosition, UniverseVar.Elements[i].Type, UniverseVar.Elements[i].RotationAngle);
                    if (6 == UniverseVar.Elements[i].Type)
                        spriteBatch.DrawString(FontList[8], String.Format("{0}", UniverseVar.Elements[i].Count), ObjScreenPosition+new Vector2(-10.0f,-10.0f), Color.Green);   // bei stationen auch den count mit reinschreiben fr erkennung
                }
                if((1 == DestinationType) && (i == DestinationIndex))
                {// der spieler hat einen station oder frachter ausgesucht also die infos zum anzeigen merken
                    ObjectVector.X = (float)(UniverseVar.Elements[i].Position_X - playerposx);
                    ObjectVector.Y = (float)(UniverseVar.Elements[i].Position_Y - playerposy);
                    mindistance = ObjectVector.LengthSquared();
                    minindex = i;
                    }
                if ((0 == DestinationType) && ((6 == UniverseVar.Elements[i].Type) || (7 == UniverseVar.Elements[i].Type)))
                {// von allen stationen und frachtern die mit der geringsten entfernung bestimmen und merken um spter den pfeil in ihre richtung zu zeichnen
                    ObjectVector.X = (float)(UniverseVar.Elements[i].Position_X - playerposx);
                    ObjectVector.Y = (float)(UniverseVar.Elements[i].Position_Y - playerposy);
                    if(ObjectVector.LengthSquared() < mindistance)
                    {
                        mindistance = ObjectVector.LengthSquared();
                        minindex = i;
                    }
                }
            }
            ObjectVector.X = (float)(UniverseVar.Elements[minindex].Position_X - playerposx);// den pfeil fr die nchste oder gewhlte station zeichnen
            ObjectVector.Y = (float)(UniverseVar.Elements[minindex].Position_Y - playerposy);
            ObjectVector.Normalize();
            ObjectVector *= -128.0f; // minus damit er in die richtige richtung zeigt (wohin ich fliegen soll) denn jetzt zeigt er noch zum schiff, zahlenwert gibt entfernung von schiffszentrum an
            stationrotation = GetRotation(ObjectVector);
            if (6 == UniverseVar.Elements[minindex].Type)
                namestring = String.Format("S{0}", UniverseVar.Elements[minindex].Count);
            else if (7 == UniverseVar.Elements[minindex].Type)
                namestring = String.Format("F{0}", UniverseVar.Elements[minindex].Count);
            DrawImage(PlayerScreenPosition - ObjectVector, 17, GetRotation(ObjectVector));   // pfeil in richtung nchster station zeichnen, - da object vector auch negiert ist fr roationsberechnung
            spriteBatch.DrawString(FontList[1], String.Format("{0}",(int)Math.Sqrt((double)mindistance)), PlayerScreenPosition-ObjectVector+new Vector2(-18.0f,-8.0f), Color.Red);  // entfernung in pfeil reinschreiben
            spriteBatch.DrawString(FontList[4], namestring, PlayerScreenPosition - ObjectVector + new Vector2(-8.0f, -65.0f), Color.Blue);  // station/frachter+nummer ber pfeil schreiben
            // den pfeil fr die bewegungsrichtung zeichnen
            ObjectVector.X = (float)(UniverseVar.Elements[PlayerNumber].Velocity_X);
            ObjectVector.Y = (float)(UniverseVar.Elements[PlayerNumber].Velocity_Y);
            if (ObjectVector.LengthSquared() > 0.001f)
            {// nur wenn bewegung ist ist vektorlnge definiert
                ObjectVector.Normalize();
                ObjectVector *= -196.0f; // minus damit er in die richtige richtung zeigt (wohin ich fliege) denn jetzt zeigt er noch zum schiff, zahlenwert gibt entfernung von schiffszentrum an
                DrawImage(PlayerScreenPosition - ObjectVector, 16, GetRotation(ObjectVector));   // pfeil in richtung bewegung zeichnen
                spriteBatch.DrawString(FontList[1], String.Format("{0:#.#}", UniverseVar.Elements[PlayerNumber].GetSpeed()), PlayerScreenPosition - ObjectVector + new Vector2(-15.0f, -8.0f), Color.Red);  // entfernung in pfeil reinschreiben
            }
            else
            {// sonst einfach den pfeil in maus- oder drehrichtung setzen und 0 anzeigen
                DrawImage(PlayerScreenPosition, 16, 0.0f);   // pfeil auf raumschiff zeichnen
                spriteBatch.DrawString(FontList[1], "0", PlayerScreenPosition+new Vector2(-6.0f,-7.0f), Color.Red);  // entfernung in pfeil reinschreiben
            }
            if (ShowMessages)
            {// die messages zeichnen
                for (i = 0; i < ((Player)(UniverseVar.Elements[PlayerNumber])).Messages.Count; i++)
                {
                    spriteBatch.DrawString(FontList[4], ((Player)(UniverseVar.Elements[PlayerNumber])).Messages[i], new Vector2(10.0f, 20.0f * (float)i), Color.LightBlue);
                }
            }
            if ((-1 != ((Player)(UniverseVar.Elements[PlayerNumber])).InsideDockingrange) && (-1 == ((Player)(UniverseVar.Elements[PlayerNumber])).Dockingindex))
                spriteBatch.DrawString(FontList[4], "Docking possible", new Vector2(370.0f, 740.0f), Color.Green);
//            spriteBatch.DrawString(FontList[4], String.Format("Player 1 Thrusters {0}  Fuel {1} money {2}", ((Player)(UniverseVar.Elements[1])).ThrustersFiring, ((Player)(UniverseVar.Elements[1])).ActFuel, ((Player)(UniverseVar.Elements[1])).Money), new Vector2(20.0f, 600.0f), Color.Red);
//            spriteBatch.DrawString(FontList[4], String.Format("Player 2 Thrusters {0}  Fuel {1} money {2}", ((Player)(UniverseVar.Elements[2])).ThrustersFiring, ((Player)(UniverseVar.Elements[2])).ActFuel, ((Player)(UniverseVar.Elements[2])).Money), new Vector2(20.0f, 630.0f), Color.Red);
            spriteBatch.End();// rendern abschlieen
            GraphicsDevice.SetRenderTarget(0, MinimapRenderTarget); // erst den nchsten rendertarget setzen bevor textur entnommen werden kann
            graphics.GraphicsDevice.DepthStencilBuffer = CreateDepthStencil(MinimapRenderTarget);  // angepassten ds-buffer setzen
            FieldmapTexture = FieldmapRenderTarget.GetTexture();
        }
        protected float GetRotation(Vector2 Targetvector)
        {// berechnet die rotation aus dem upvektor und den zielvektor
            float rotation = (float)Math.Acos((double)((Upvector.X * Targetvector.X + Upvector.Y * Targetvector.Y) / (Upvector.Length() * Targetvector.Length())));   // rotationswinkel aus skalarprodukt der vektoren (upvektor) berechnen
            if (Targetvector.X < 0.0f)
                return rotation;
            else
                return rotation * -1.0f;
        }
        protected void RenderMinimap()
        {// rendert die minimap in eine textur
            int type;   // die nummer des bildes im texture-array je nach objekttyp
            float rotation, radius = (float)UniverseVar.Radius; // die rotation wird nur fr spielerschiffe genutzt
            Vector2 position = new Vector2(), offset = new Vector2(-4.0f, -7.0f), playerpos = new Vector2(0.0f,0.0f);
            spriteBatch.Begin();
            GraphicsDevice.Clear(Color.DarkOliveGreen);
            //# minimap rendern
            DrawImage(new Vector2(128.0f, 128.0f), 10, 0.0f);   // kreis zeichnen (der selbe wie docking kreis)
            type = 0;
            position.X = 0.0f;
            position.Y = 0.0f;
            for (int i = 0; i < UniverseVar.Elements.Count; i++)
            {
                rotation = 0.0f;
                if(UniverseVar.Elements[i].Type < 6)    // asteroid, alle typen gleich dargestellt
                    type = 11;
                if(6 == UniverseVar.Elements[i].Type)    // station
                    type = 15;
                if(7 == UniverseVar.Elements[i].Type)    // frachter
                    type = 13;
                if((8 == UniverseVar.Elements[i].Type) && (i != PlayerNumber))   // anderer player
                {
                    type = 12;
                    rotation = UniverseVar.Elements[i].RotationAngle;
                }
                if ((8 == UniverseVar.Elements[i].Type) && (i == PlayerNumber)) // player
                {
                    type = 14;
                    rotation = UniverseVar.Elements[i].RotationAngle;
                }
                position.X = ((float)UniverseVar.Elements[i].Position_X + radius) / (radius * 2.0f) * 256.0f;
                position.Y = ((float)UniverseVar.Elements[i].Position_Y + radius) / (radius * 2.0f) * 256.0f;
                if (i == PlayerNumber)
                    playerpos = position;   // die playerpos merken fr screenrect zeichnen
                DrawImage(position, type, rotation);
                if ((6 == UniverseVar.Elements[i].Type) || (7 == UniverseVar.Elements[i].Type))    // station und frachter hier die nummer reinrendern
                    spriteBatch.DrawString(FontList[0], String.Format("{0}", UniverseVar.Elements[i].Count), position+offset, Color.Red);
            }
            float scale = (768.0f)/(float)UniverseVar.Radius;   // das verhltnis des sichtbaren bereiches zum gesamtbereich (spielfeld) ausrechnen fr skalierung
            playerpos.X = (float)Math.Floor(playerpos.X);
            playerpos.Y = (float)Math.Floor(playerpos.Y);
            spriteBatch.Draw(TextureList[18], playerpos, null, Color.White, 0.0f, new Vector2(TextureList[18].Height / 2, TextureList[18].Height / 2), scale, SpriteEffects.None, 0.0f);
            spriteBatch.End();
            GraphicsDevice.SetRenderTarget(0, InfoscreenRenderTarget); // erst den nchsten rendertarget setzen bevor textur entnommen werden kann
            graphics.GraphicsDevice.DepthStencilBuffer = CreateDepthStencil(FieldmapRenderTarget);  // angepassten ds-buffer setzen
            MinimapTexture = MinimapRenderTarget.GetTexture();
        }
        protected void RenderInfoscreen()
        {// rendert den infoscreen in eine textur
            spriteBatch.Begin();
            GraphicsDevice.Clear(Color.DarkSlateGray);
            Player player = (Player)UniverseVar.Elements[PlayerNumber],tempplayer;
            Freighter tempfreighter;
            Station station, tempstation;
            float pixx = 5.0f, pixy = 5.0f, linehight = 15.0f, actline;
            int maxmoney=0, i, amount,totalprice;
            for (i = 0; i < UniverseVar.NumberofPlayers; i++)
            {// alle spieler durchgehen um das meiste geld zu bestimmen fr vergleich
                if (((Player)(UniverseVar.Elements[i])).Money > maxmoney)
                    maxmoney = ((Player)(UniverseVar.Elements[i])).Money;
            }
            spriteBatch.DrawString(FontList[2], String.Format("Time to go: {0:00}:{1:00}:{2:00}", UniverseVar.TimeToRun.Hours, UniverseVar.TimeToRun.Minutes, UniverseVar.TimeToRun.Seconds), new Vector2(pixx, pixy + 0.0f * linehight), Color.White);    // hier zeit schreiben
            spriteBatch.DrawString(FontList[2], String.Format("Money: {0}  Best: {1}", player.Money, maxmoney), new Vector2(pixx, pixy + 1.0f * linehight), Color.White);
            spriteBatch.DrawString(FontList[2], String.Format("Armor: {0} / {1}", player.ActHitpoints, player.MaxHitpoints()), new Vector2(pixx, pixy + 2.0f * linehight), Color.White);
            spriteBatch.DrawString(FontList[2], String.Format("Fuel: {0:0} / {1}", player.ActFuel, player.MaxFuel()), new Vector2(pixx, pixy + 3.0f * linehight), Color.White);
            spriteBatch.DrawString(FontList[2], String.Format("Cargo: {0} / {1}", player.LoadedOre+player.LoadedSupply, player.MaxCargoCapacity()), new Vector2(pixx, pixy + 4.0f * linehight), Color.White);
            spriteBatch.DrawString(FontList[2], String.Format("  Ore: {0} Supply: {1}", player.LoadedOre, player.LoadedSupply), new Vector2(pixx, pixy + 5.0f * linehight), Color.White);
            if (-1 < player.Dockingindex)
            {// wenn an station oder frachter gedockt kann man immer erz und supply kaufen und verkaufen
                if (6 == UniverseVar.Elements[player.Dockingindex].Type)
                {// an Station gedockt
                    tempplayer = new Player(player);    // kopie erzeugen um zu handeln und die tatschlichen (aktualisierten) werte ausgeben zu geben zb wenn ich mit durch verkauf verdientem geld mehr einkaufen kann
                    tempstation = new Station((Station)(UniverseVar.Elements[player.Dockingindex]));
                    spriteBatch.DrawString(FontList[2], "Trade [T]", new Vector2(pixx, pixy + 6.0f * linehight), Color.White);
                    player.GetSellSupplyData(out amount, out totalprice, tempstation);
                    spriteBatch.DrawString(FontList[2], String.Format("  -{0} Sup for +{1} Money", amount, totalprice), new Vector2(pixx, pixy + 7.0f * linehight), Color.White);//supply verkaufen
                    tempplayer.Trade(tempstation, out amount, out totalprice);  // virtuell verkaufen um die daten frs kaufen zu kriegen fr anzeige
                    spriteBatch.DrawString(FontList[2],String.Format("  +{0} Ore for -{1} Money", amount, totalprice), new Vector2(pixx, pixy + 8.0f * linehight), Color.White);//erz kaufen
                    if ((1==UniverseVar.StationRepairs) && (player.MaxHitpoints() > player.ActHitpoints))
                    {
                        player.GetRepairData(out amount, out totalprice, tempstation.GetRepairPrice());
                        spriteBatch.DrawString(FontList[2], "Repair  [R]", new Vector2(pixx, pixy + (float)Math.Floor(9.5f * linehight)), Color.White);
                        spriteBatch.DrawString(FontList[2], String.Format("  +{0} Units for -{1} Money", amount, totalprice), new Vector2(pixx, pixy + (float)Math.Floor(10.5f * linehight)), Color.White);//schiff reparieren
                    }
                    if ((1 == UniverseVar.StationRefuels) && (player.MaxFuel() > player.ActFuel))
                    {
                        player.GetRefuelData(out amount, out totalprice, tempstation.GetFuelPrice());
                        spriteBatch.DrawString(FontList[2], "Refuel  [F]", new Vector2(pixx, pixy + (float)Math.Floor(11.5f * linehight)), Color.White);
                        spriteBatch.DrawString(FontList[2], String.Format("  +{0} Units for -{1} Money", amount, totalprice), new Vector2(pixx, pixy + (float)Math.Floor(12.5f * linehight)), Color.White);//schiff betanken
                    }
                    if (1 == UniverseVar.StationSellsUpgrades)
                    {// wenn upgrades verkauft werden diese alle einzeln anzeigen
                        player.GetAgilUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Agility Upgrade [A]", new Vector2(pixx, pixy + 14.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 15.0f * linehight), Color.White);
                        }
                        player.GetBrakeUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Brake Upgrade [X]", new Vector2(pixx, pixy + 16.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 17.0f * linehight), Color.White);
                        }
                        player.GetHitpUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Armor Upgrade [P]", new Vector2(pixx, pixy + 18.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 19.0f * linehight), Color.White);
                        }
                        player.GetCargoUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Cargohold Upgrade [L]", new Vector2(pixx, pixy + 20.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 21.0f * linehight), Color.White);
                        }
                        player.GetFuelUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Fuel Upgrade [K]", new Vector2(pixx, pixy + 22.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 23.0f * linehight), Color.White);
                        }
                        player.GetThrustUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Thruster Upgrade [T]", new Vector2(pixx, pixy + 24.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 25.0f * linehight), Color.White);
                        }
                    }
                }
                else
                {// an frachter gedockt,
                    tempplayer = new Player(player);    // kopie erzeugen um zu handeln und die tatschlichen (aktuelisierten) werte ausgeben zu geben zb wenn ich mit durch verkauf verdientem geld mehr einkaufen kann
                    tempfreighter = new Freighter((Freighter)(UniverseVar.Elements[player.Dockingindex]));
                    spriteBatch.DrawString(FontList[2], "Trade [T]", new Vector2(pixx, pixy + 6.0f * linehight), Color.White);
                    player.GetSellOreData(out amount, out totalprice, tempfreighter);
                    spriteBatch.DrawString(FontList[2], String.Format("  -{0} Ore for +{1} Money", amount, totalprice), new Vector2(pixx, pixy + 7.0f * linehight), Color.White);//erz verkaufen
                    tempplayer.Trade(tempfreighter, out amount, out totalprice);  // virtuell verkaufen um die daten frs kaufen zu kriegen fr anzeige
                    spriteBatch.DrawString(FontList[2], String.Format("  +{0} Sup for -{1} Money", amount, totalprice), new Vector2(pixx, pixy + 8.0f * linehight), Color.White);//supply kaufen
                    if ((1 == UniverseVar.FreighterRepairs) && (player.MaxHitpoints() > player.ActHitpoints))
                    {
                        player.GetRepairData(out amount, out totalprice, tempfreighter.GetRepairPrice());
                        spriteBatch.DrawString(FontList[2], "Repair  [R]", new Vector2(pixx, pixy + (float)Math.Floor(9.5f * linehight)), Color.White);
                        spriteBatch.DrawString(FontList[2], String.Format("  +{0} Units for -{1} Money", amount, totalprice), new Vector2(pixx, pixy + (float)Math.Floor(10.5f * linehight)), Color.White);//schiff reparieren
                    }
                    if ((1 == UniverseVar.FreighterRefuels) && (player.MaxFuel() > player.ActFuel))
                    {
                        player.GetRefuelData(out amount, out totalprice, tempfreighter.GetFuelPrice());
                        spriteBatch.DrawString(FontList[2], "Refuel  [F]", new Vector2(pixx, pixy + (float)Math.Floor(11.5f * linehight)), Color.White);
                        spriteBatch.DrawString(FontList[2], String.Format("  +{0} Units for -{1} Money", amount, totalprice), new Vector2(pixx, pixy + (float)Math.Floor(12.5f * linehight)), Color.White);//schiff betanken
                    }
                    if (1 == UniverseVar.FreighterSellsUpgrades)
                    {// wenn upgrades verkauft werden diese alle einzeln anzeigen
                        player.GetAgilUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Agility Upgrade [A]", new Vector2(pixx, pixy + 14.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 15.0f * linehight), Color.White);
                        }
                        player.GetBrakeUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Brake Upgrade [X]", new Vector2(pixx, pixy + 16.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 17.0f * linehight), Color.White);
                        }
                        player.GetHitpUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Armor Upgrade [P]", new Vector2(pixx, pixy + 18.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 19.0f * linehight), Color.White);
                        }
                        player.GetCargoUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Cargohold Upgrade [L]", new Vector2(pixx, pixy + 20.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 21.0f * linehight), Color.White);
                        }
                        player.GetFuelUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Fuel Upgrade [K]", new Vector2(pixx, pixy + 22.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 23.0f * linehight), Color.White);
                        }
                        player.GetThrustUpgrData(out amount, out totalprice);
                        if ((amount < player.MaxUpgradeLevel) && (totalprice <= player.Money))
                        {
                            spriteBatch.DrawString(FontList[2], "Buy Thruster Upgrade [T]", new Vector2(pixx, pixy + 24.0f * linehight), Color.White);
                            spriteBatch.DrawString(FontList[2], String.Format("{0} / {1} for -{2} Money", amount, player.MaxUpgradeLevel, totalprice), new Vector2(pixx, pixy + 25.0f * linehight), Color.White);
                        }
                    }
                }
            }
            actline = 385.0f;
            for (i = 0; i < UniverseVar.Elements.Count; i++)
            {// status aller stationen schreiben, das in einer zeile machen da sonst zu platzinztensiv (schriftgre, S1 O:15 S:7 E:45)
                if (6 == UniverseVar.Elements[i].Type)
                {
                    actline += linehight;
                    station = (Station)UniverseVar.Elements[i];
                    spriteBatch.DrawString(FontList[1], String.Format("S{0} O {1} S {2} E {3}", station.Count,station.Ore,station.Supply,UniverseVar.StationCargoHold - station.Ore - station.Supply), new Vector2(pixx, actline+0.0f*linehight), Color.White);
                }
            }
            spriteBatch.End();
            GraphicsDevice.SetRenderTarget(0, null);    // rendertarget wieder auf screen setzen
            graphics.GraphicsDevice.DepthStencilBuffer = oldBuffer; 
            InfoscreenTexture=InfoscreenRenderTarget.GetTexture();
        }
        protected DepthStencilBuffer CreateDepthStencil(RenderTarget2D target)
        {// entnommen von: http://msdn.microsoft.com/en-us/library/microsoft.xna.framework.graphics.depthstencilbuffer.aspx
            return new DepthStencilBuffer(target.GraphicsDevice, target.Width,target.Height, target.GraphicsDevice.DepthStencilBuffer.Format,target.MultiSampleType, target.MultiSampleQuality);
        }


    }


    static class Program
    {
        static void Main(string[] args)
        {// The main entry point for the application.
            try
            {
                RemotingConfiguration.Configure("FSDSD_Client.exe.config", false);
                using (FSDSD_Game game = new FSDSD_Game())
                {
                    game.Run();
                }
            }
            catch (Exception e)
            {
                System.Windows.Forms.MessageBox.Show(e.ToString());
            }
        }
    }
}



