﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
    
[Serializable]

public class FSDSD_Server_Interface:MarshalByRefObject
{
    public int NumberofConnectedClients;    // anzahl der clients die sich verbunden haben
    public int RandomSeed;  // seed für den zufallszahlengenerator
    public int NumberofExpectedClients;     // anzahl der spieler (clients) die zum spielen benötigt werden (1 für singleplayer)
    public int AllowedServerRequests;   // die maximale zahl an abfragen der clients an den server pro minute, client fragt jedes frame ob er schon wieder darf aber server limitiert die zahl für fairness und trafficeinsparung

    public double TimeDiff;     // die Zeit zwischen zwei client-abfragen in millisekunden
    public Random RandomNumberGenerator;
    public Universe UniverseVar, UniverseCopy;
    public float MinSpawnDistance;
    public bool Locked, GameOver; // speichert ob ein zugriff möglich ist oder ob gerade eine kopieraktion (thread safe) im gange ist, dann anfragen zurückweisen
    public DateTime LastUpdateTime;    // zeit von letztem update
    public int LastUpdateMinute;

    public FSDSD_Server_Interface()
    {
        UniverseVar = new Universe();
        UniverseCopy = new Universe();
        NumberofConnectedClients = 0;
        UniverseVar.GamePaused = true;  // am anfang pausiert bis geforderte anzahl clients verbunden hat
        Locked = GameOver = false;
        LastUpdateMinute = -1;
        ReadGameSettingsFile();
        MinSpawnDistance = 128.0f;
        TimeDiff = 60000.0 / (double)AllowedServerRequests;
        UniverseVar.TimeToRun = TimeSpan.FromMinutes((double)UniverseVar.GameDuration);
        GenerateGame();
        for (int i = 0; i < NumberofExpectedClients; i++)
            ((Player)(UniverseVar.Elements[i])).AddMessage("Game created");
        UniverseCopy.MakeCopy(UniverseVar); // kopiere das erstellte spiel
        LastUpdateTime = DateTime.Now;
    }
    ~FSDSD_Server_Interface()
    {
    }
    public Universe GetUniverse()
    {// return the array with all asteroids, ships + stations
        if (Locked == true) // wenn gerade kopiert wird null zurückgeben was client veranlasst weiterhin abzufragen
            return null;
        return UniverseCopy;    // wenn alles ok die kopie zurückgeben
    }
    public int RegisterClientwithServer()
    {// funktion die client einmal aufruft um initial kontakt mit dem server aufzunehmen und zu prüfen ob er berechtigt ist (spielerzahl)
        //# hier noch den namen abfragen und in player class speichern
        if (NumberofExpectedClients == NumberofConnectedClients)
        {// Teilnehmerzahl schon erreicht also dem client den eintritt verwehren
            Console.WriteLine("No more Clients allowed. Adjust GameSettings.ini and restart the Server.");
            return -1;  // wert führt bei Client zu fehlermeldung und beendigung
        }
        Console.WriteLine("Connected Clients: {0} / {1}", ++NumberofConnectedClients, NumberofExpectedClients);
        for (int i = 0; i < NumberofExpectedClients; i++)   // die message queue befüllen
            ((Player)(UniverseVar.Elements[i])).AddMessage(String.Format("Connected Players {0} / {1}", NumberofConnectedClients, NumberofExpectedClients));
        if (NumberofExpectedClients == NumberofConnectedClients)
        {// wenn nach dem inkrement die benötigte anzahl erreicht wurde die pause aufheben und spiel starten
            UniverseVar.GamePaused = false;
            Console.WriteLine("All Clients connected. Game running.");
            for (int i = 0; i < NumberofExpectedClients; i++)   // die message queue befüllen
                ((Player)(UniverseVar.Elements[i])).AddMessage("All Clients connected. Game running.");
        }
        return NumberofConnectedClients - 1;    // return value ist die playernumber mit der sich dann jeder client identifiziert, die spieler ids starten bei 0 und je nach reihenfolge der anmeldung werden die spieler verteilt (farben), -1 damit index 0 basiert ist für direkten zugriff in liste
    }
    public int CheckUpdateState(int playernumber)
    {// client fragt jedes frame ob er schon neue daten holen darf (0 - nein sonst ja) um zu viel traffic zu vermeiden und spieler mit schlechter verbindung nicht zu benachteiligen
        DateTime nowtime = DateTime.Now;
        TimeSpan lastupdate = nowtime.Subtract(LastUpdateTime);
        if (lastupdate.Seconds*1000.0f + lastupdate.Milliseconds > 15)
        {// 60 mal pro sekunde update machen (objekte bewegen, ki, kollision)
            UpdateUniverse((float)(lastupdate.Seconds * 1000 + lastupdate.Milliseconds) / 1000.0f);
            LastUpdateTime = nowtime;
        }
        if (nowtime > ((Player)UniverseVar.Elements[playernumber]).LastActionTime.AddMilliseconds(TimeDiff))
        {// wenn genügen zeit verstrichen ist seit letztem update sende ok (1)
            ((Player)UniverseVar.Elements[playernumber]).LastActionTime = nowtime;
            return 1;
        }
        return 0;   // noch nicht genug zeit verstrichen
    }
    public void PauseGame(int playernumber)
    {// spiel wird pausiert wenn nur ein spieler
        if (1 == NumberofExpectedClients)
        {// spiel unterbrechen
            if (UniverseVar.GamePaused)
            {// pause aufheben
                if (false == GameOver)
                {
                    UniverseVar.GamePaused = false;
                    ((Player)(UniverseVar.Elements[0])).AddMessage("Game running. [PAUSE] to pause.");
                }
                else
                    ((Player)(UniverseVar.Elements[0])).AddMessage("Game is over and cannot be resumed.");
            }
            else
            {// pause starten
                UniverseVar.GamePaused = true;
                ((Player)(UniverseVar.Elements[0])).AddMessage("Game paused. [PAUSE] to resume.");
            }
        }
        else
        {
            ((Player)(UniverseVar.Elements[playernumber])).AddMessage("Pause in Multiplayergame not possible. Dock somewhere.");
        }
    }
    public void FireThrusters(int playernumber)
    {// der Antrieb des Spielerschiffes wird gezündet
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 <= player.Dockingindex)   // wenn spieler gedockt nicht beschleunigen
            return;
        if (UniverseVar.FuelUsage * player.ActAcceleration() > player.ActFuel)
            player.AddMessage("You ran out of Fuel. Press [ESC] to give up an end Game.");
        else if ((false == player.ThrustersFiring) && (UniverseVar.FuelUsage*player.ActAcceleration() <= player.ActFuel))
        {// die triebwerke werden zum ersten mal gezündet (oder erneut nach ablauf des letzten zykles)
            player.ThrustersFiring = true;
            player.ThrusterTime = DateTime.Now; // zeit merken
            player.ActFuel -= UniverseVar.FuelUsage * player.ActAcceleration();   // treibstoff abziehen
        }
    }
    public void ClientTerminates(int playernumber)
    {// ein Spieler hat aufgegeben
        Player player = (Player)UniverseVar.Elements[playernumber];
        player.Lost = true;
        player.Velocity_X = 0.0f;
        player.Velocity_Y = 0.0f;
        for (int i = 0; i < NumberofExpectedClients; i++)   // die message queue befüllen
            ((Player)(UniverseVar.Elements[i])).AddMessage(String.Format("Player {0} has give up.", playernumber + 1));
    }
    public int Docking(int playernumber)
    {// behandelt an- und abdockanforderungen
        Player player = (Player)UniverseVar.Elements[playernumber];
        if(player.Dockingindex >= 0)
        {// schiff ist gedockt also abdocken
            if(6 == UniverseVar.Elements[player.Dockingindex].Type)
                player.AddMessage(String.Format("Undocked from Station {0}", UniverseVar.Elements[player.Dockingindex].Count));
            else if (7 == UniverseVar.Elements[player.Dockingindex].Type)
                player.AddMessage(String.Format("Undocked from Freighter {0}", UniverseVar.Elements[player.Dockingindex].Count));
            player.Dockingindex = -1;
            player.DockedPos_X = 0.0f;  // die relativposition zurücksetzen
            player.DockedPos_Y = 0.0f;
        }
        else
        {// schiff möchte andocken also an das in reichweite docken lassen
            if (0 > player.InsideDockingrange)
            {
                player.AddMessage("Docking not available. Nothing in Range.");
                return -1;  // nichts innerhalb dockingrange
            }
            player.Dockingindex = player.InsideDockingrange;
            player.DockedPos_X = player.Position_X - UniverseVar.Elements[player.Dockingindex].Position_X;  // die relativposition zum dockingzeitpunkt speichern
            player.DockedPos_Y = player.Position_Y - UniverseVar.Elements[player.Dockingindex].Position_Y;
            player.Velocity_X = player.Velocity_Y = 0.0f;   // wenn andockt geschwindigkeit wegnehmen
            player.ThrustersFiring = false; // triebwerke ausschalten (obwohl das auch automatisch nach einiger zeit (< 1 sec) gemacht wird)
            if (6 == UniverseVar.Elements[player.Dockingindex].Type)
                player.AddMessage(String.Format("Docked at Station {0}",UniverseVar.Elements[player.Dockingindex].Count));
            else if (7 == UniverseVar.Elements[player.Dockingindex].Type)
                player.AddMessage(String.Format("Docked at Freighter {0}", UniverseVar.Elements[player.Dockingindex].Count));
        }
        return 0;   // keine station zum andocken gefunden
    }
    public void Brake(int playernumber)
    {// bremst das schiff ab
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (player.Dockingindex >= 0)
            return; // spieler ist gedockt also verlassen
        if (false == player.ArtificIntelligence)
        {// ki spieler verbrauchen fürs bremsen keinen treibstoff da sie durchaus mehrfach hintereinander bremsen können (was aber bei 0 gesch auch nichts verbraucht),
            if (player.ActFuel < player.GetSpeed() * player.ActBrake() * 3.0f)
            {
                player.AddMessage("Not enough Fuel for Brake.");
                return;
            }
            player.ActFuel -= player.GetSpeed() * player.ActBrake() * 3.0f; // treibstoff abziehen
        }
        player.Velocity_X *= player.ActBrake();
        player.Velocity_Y *= player.ActBrake();
    }
    public void UpdateRotation(int playernumber, float targetx, float targety)
    {// bestimmt in welche richtung sich das schiff rotieren soll die berechnung selbst wird auf server gemacht
        ((Player)(UniverseVar.Elements[playernumber])).Targetrotation = GetRotation(targetx, targety);
    }
    public void Trade(int playernumber)
    {// wo der spieler gedockt ist (wenn) möchte er handeln (das begehrte verkaufen und das vorhandene kaufen)
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)    // player ist nicht gedockt
        {
            player.AddMessage("You must be docked for trading");
            return;
        }
        int t1, t2;
        player.Trade(UniverseVar.Elements[player.Dockingindex],out t1, out t2); // rückgabewerte hier ignorieren
    }
    public void BuyCargoUpgrade(int playernumber)
    {// kauft das cargoupgrade für diesen spieler
        Player player = (Player)UniverseVar.Elements[playernumber];
        if(0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to buy Upgrades.");
            return;
        }
        if(((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationSellsUpgrades)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterSellsUpgrades)))
            player.BuyCargoUpgrade();
        else
            player.AddMessage("You cannot buy an Upgrade here (change Serversettings).");
    }
    public void BuyFuelUpgrade(int playernumber)
    {// kauft das fuelupgrade (tankkapazität) für diesen spieler
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to buy Upgrades.");
            return;
        }
        if (((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationSellsUpgrades)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterSellsUpgrades)))
            player.BuyFuelUpgrade();
        else
            player.AddMessage("You cannot buy an Upgrade here (change Serversettings).");
    }
    public void BuyHitpointUpgrade(int playernumber)
    {// kauft das panzerungsupgrade für diesen spieler
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to buy Upgrades.");
            return;
        }
        if (((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationSellsUpgrades)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterSellsUpgrades)))
            player.BuyHitpointUpgrade();
        else
            player.AddMessage("You cannot buy an Upgrade here (change Serversettings).");
    }
    public void BuyAgilityUpgrade(int playernumber)
    {// kauft das wendigkeitsupgrade für diesen spieler
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to buy Upgrades.");
            return;
        }
        if (((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationSellsUpgrades)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterSellsUpgrades)))
            player.BuyAgilityUpgrade();
        else
            player.AddMessage("You cannot buy an Upgrade here (change Serversettings).");
    }
    public void BuyAccelerationUpgrade(int playernumber)
    {// kauft das beschleunigungssupgrade für diesen spieler
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to buy Upgrades.");
            return;
        }
        if (((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationSellsUpgrades)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterSellsUpgrades)))
            player.BuyAccelerationUpgrade();
        else
            player.AddMessage("You cannot buy an Upgrade here (change Serversettings).");
    }
    public void BuyBrakeUpgrade(int playernumber)
    {// kauft das bremsenupgrade für diesen spieler
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to buy Upgrades.");
            return;
        }
        if (((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationSellsUpgrades)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterSellsUpgrades)))
            player.BuyBrakeUpgrade();
        else
            player.AddMessage("You cannot buy an Upgrade here (change Serversettings).");
    }
    public void Repair(int playernumber)
    {// repariert das schiff
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to Repair your ship.");
            return;
        }
        if (((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationRepairs)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterRepairs)))
            player.Repair(UniverseVar.Elements[player.Dockingindex], UniverseVar.RepairCost);
        else
            player.AddMessage("You cannot Repair here (change Serversettings).");
    }
    public void Refuel(int playernumber)
    {// betankt das schiff
        Player player = (Player)UniverseVar.Elements[playernumber];
        if (0 > player.Dockingindex)
        {
            player.AddMessage("You must be docked to Refuel.");
            return;
        }
        if (((6 == UniverseVar.Elements[player.Dockingindex].Type) && (1 == UniverseVar.StationRefuels)) || ((7 == UniverseVar.Elements[player.Dockingindex].Type) & (1 == UniverseVar.FreighterRefuels)))
            player.Refuel(UniverseVar.Elements[player.Dockingindex], UniverseVar.FuelCost);
        else
            player.AddMessage("You cannot Refuel here (change Serversettings).");
    }

    private void ReadGameSettingsFile()
    {
        try
        {
            StreamReader reader = File.OpenText(Path.GetDirectoryName(Assembly.GetEntryAssembly().Location) + "\\GameSettings.ini"); // Datei öffnen
            string[] lines;
            lines = reader.ReadLine().Split('\t');
            RandomSeed = int.Parse(lines[0]);
            if (0 == RandomSeed)    // kein seed gesetzt also zufällig initialisieren
                RandomNumberGenerator = new Random();
            else    // seed gesetzt also damit initialisieren
                RandomNumberGenerator = new Random(RandomSeed);
            //# werte größer/kleiner min/max anpassen,
            lines = reader.ReadLine().Split('\t');
            UniverseVar.Radius = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.NumberofAsteroids = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.NumberofStations = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.NumberofFreighters = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.StationProductionRate = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.StationSupplyUsageRate = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.StationCargoHold = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.StationSellsUpgrades = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.StationRepairs = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.StationRefuels = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.FreighterSellsUpgrades = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.FreighterRepairs = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.FreighterRefuels = int.Parse(lines[0]); 
            lines = reader.ReadLine().Split('\t');
            UniverseVar.FuelCost = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.RepairCost = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            NumberofExpectedClients = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.NumberofPlayers = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.GameDuration = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.UpgradeCostFactor = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.UpgradeCostBase = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.FuelUsage = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            AllowedServerRequests = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.MaxAsteroidVelocity = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.FuelAmount = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.MaxHitpoints = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.MaxPlayerCargo = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.MaxUpgradeLevel = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.MaxMessages = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.SupplyMinPrice = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.SupplyMaxPrice = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.OreMinPrice = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.OreMaxPrice = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.PriceBonus = int.Parse(lines[0]);
            lines = reader.ReadLine().Split('\t');
            UniverseVar.StartMoney = int.Parse(lines[0]);
            reader.Close();
        }
        catch (FileNotFoundException)
        {
            Console.WriteLine("GameSettings could ne be loaded. Missing or corrupt <GameSettings.ini>.");
            Environment.Exit(0);
        }
        Console.WriteLine("GameSettings succesfully loaded");
    }
    private void GenerateGame()
    {// belegt die asteroiden, stationen, frachter, spieler mit positionen
        int  temptype,i, count;
        float tempposx, tempposy, tempvelx, tempvely,rotang, rotspd;
        count = 0;
        for (i = 0; i < UniverseVar.NumberofPlayers; i++)
        {// spieler erzeugen und platzieren
            GenerateValidPosition(out tempposx, out tempposy, true);
            UniverseVar.Elements.Add(new Player(tempposx, tempposy, 0.0f, 0.0f, 0.0f, 0.0f, 8, ++count, false, UniverseVar.FuelAmount, UniverseVar.MaxHitpoints, UniverseVar.MaxPlayerCargo, UniverseVar.UpgradeCostFactor, UniverseVar.UpgradeCostBase, UniverseVar.MaxUpgradeLevel, UniverseVar.MaxMessages,UniverseVar.StartMoney));   //# evtl noch zum zentrum rotieren
        }
        count = 0;
        for (i = 0; i < UniverseVar.NumberofFreighters; i++)
        {// Frachter erzeugen und platzieren
            GenerateValidPosition(out tempposx, out tempposy, true);
            UniverseVar.Elements.Add(new Freighter(tempposx, tempposy, 0.0f, 0.0f, 0.0f, 0.0f, 7, ++count, true, UniverseVar.RepairCost, UniverseVar.FuelCost, UniverseVar.OreMaxPrice + UniverseVar.PriceBonus, UniverseVar.SupplyMinPrice - UniverseVar.PriceBonus));   //# evtl noch zum zentrum rotieren
        }
        count = 0;
        for (i = 0; i < UniverseVar.NumberofStations; i++)
        {// Stationen erzeugen und platzieren
            GenerateValidPosition(out tempposx, out tempposy, false);
            tempvelx = ((float)RandomNumberGenerator.NextDouble() * (float)UniverseVar.MaxAsteroidVelocity * 2.0f) - UniverseVar.MaxAsteroidVelocity;
            tempvely = ((float)RandomNumberGenerator.NextDouble() * (float)UniverseVar.MaxAsteroidVelocity * 2.0f) - UniverseVar.MaxAsteroidVelocity;
            rotang = (float)RandomNumberGenerator.NextDouble() * 6.28f - 3.14f;   // startwert für winkel
            rotspd = ((float)RandomNumberGenerator.NextDouble() - 0.5f);  // geschwindigkeit für winkel (einheit?)
            UniverseVar.Elements.Add(new Station(tempposx, tempposy, tempvelx, tempvely, rotang, rotspd, 6, ++count, true, UniverseVar.StationCargoHold, UniverseVar.RepairCost, UniverseVar.FuelCost, UniverseVar.SupplyMaxPrice, UniverseVar.SupplyMinPrice, UniverseVar.OreMaxPrice, UniverseVar.OreMinPrice));
        }
        count = 0;
        for (i = 0; i < UniverseVar.NumberofAsteroids; i++)
        {// asteroiden erzeugen und platzieren
            GenerateValidPosition(out tempposx, out tempposy, false);
            tempvelx = ((float)RandomNumberGenerator.NextDouble() * (float)UniverseVar.MaxAsteroidVelocity * 2.0f) - UniverseVar.MaxAsteroidVelocity;
            tempvely = ((float)RandomNumberGenerator.NextDouble() * (float)UniverseVar.MaxAsteroidVelocity * 2.0f) - UniverseVar.MaxAsteroidVelocity;
            rotang = (float)RandomNumberGenerator.NextDouble() * 6.28f - 3.14f;   // startwert für winkel
            rotspd = ((float)RandomNumberGenerator.NextDouble() -0.5f);  // geschwindigkeit für winkel (einheit?)
            temptype = RandomNumberGenerator.Next(4)+1;   // 5 typen von asteroiden (bilder) alle gleiche eigenschaften
            UniverseVar.Elements.Add(new Asteroid(tempposx, tempposy, tempvelx, tempvely, rotang, rotspd, temptype, ++count, false));
        }
        for (i = NumberofExpectedClients; i < UniverseVar.NumberofPlayers; i++)
            ((Player)(UniverseVar.Elements[i])).ArtificIntelligence = true; // den player auf ki setzen fürs "cheaten"
        Console.WriteLine("Game succesfully created.");
    }
    private void GenerateValidPosition(out float posx, out float posy, bool border)
    {// erzeugt eine position die mindestens einen kollisionsabstand von allem weg ist
        int iterations = 0, maxiterations=10000,i;
        bool iterateagain = true;
        float distx, disty, tempx = 0.0f, tempy = 0.0f,radius = (float)UniverseVar.Radius * 0.98f;  // sicherstellen das alle innerhalb des radius starten
        while (iterateagain)
        {
            if (++iterations > maxiterations)
            {// sicherheitsabfrage damit erzeugung nicht ewig dauert
                Console.WriteLine("GenerateValidPosition: Over {0} Iterations without a valid Position. Reduce objects or increase size (radius). Shut down Server with <Enter>", maxiterations);
                Console.ReadLine();
                Environment.Exit(0);
            }
            if (border)
            {// es soll eine position direkt am rand erzeugt werden (für spieler + frachter)
                tempx = (float)RandomNumberGenerator.NextDouble() * radius * 2.0f - radius;  // eine zufällige x-koordinate erzeugen
                tempy = (float)Math.Sqrt((double)(radius * radius - tempx * tempx));
                if (1 == RandomNumberGenerator.Next(2))  // in 50% der fälle den negativen fall wählen
                    tempy *= -1.0f;
            }
            else
            {// eine position mittendrin (asteroiden + stationen)
                do
                {
                    tempx = (float)RandomNumberGenerator.NextDouble() * radius * 2.0f - radius;
                    tempy = (float)RandomNumberGenerator.NextDouble() * radius * 2.0f - radius;
                } while (tempx * tempx + tempy * tempy > radius * radius);  // checken ob der punkt im kreis liegt
            }
            iterateagain = false;
            for (i = 0; i < UniverseVar.Elements.Count; i++)
            {// für jedes schon vorhandene Element prüfen
                distx = UniverseVar.Elements[i].Position_X - tempx;
                disty = UniverseVar.Elements[i].Position_Y - tempy;
                if (distx * distx + disty * disty < MinSpawnDistance * MinSpawnDistance)    // ein anderes objekt liegt zu nahe, also koordinaten nochmal erzeugen
                    iterateagain = true;
            }
        }
        posx = tempx;
        posy = tempy;
    }
    private void UpdateUniverse(float secondfraction)
    {// führt alle aktualisierungen (bewegung, kollision) durch, playereingaben werden sofort aktualisiert
        int i;
        DateTime nowtime = DateTime.Now;
        if (false == UniverseVar.GamePaused) // wenn game pausiert ist nur kopie machen
        {
            Player player;
            float dx, dy, alpha, pi = (float)Math.PI; // die beschleunigung für diesen triebwerkszyklus
            for (i = 0; i < UniverseVar.NumberofPlayers; i++)
            {// prüfen ob triebwerke deaktiviert werden müssen
                player = (Player)UniverseVar.Elements[i];
                if ((true == player.ThrustersFiring) && (nowtime > player.ThrusterTime.AddSeconds(0.5)))
                {// die triebwerke brennen noch schauen ob sie abgeschaltet werden müssen (1 sec brenndauer)
                    player.ThrustersFiring = false;
                    // die beschleunigung in aktuelle richtung ausführen, dabei wird einheitsvektor (länge=1 berechnet was der beschleunigung um 1 in diese richtung entspricht)
                    if (player.RotationAngle > pi * 0.5f)
                        alpha = pi - player.RotationAngle;
                    else if (player.RotationAngle < pi * -0.5f)
                        alpha = pi * -1.0f - player.RotationAngle;
                    else // -pi/2 < rotation < pi/2
                        alpha = player.RotationAngle;
                    dx = (float)Math.Sin((double)alpha);
                    dy = (float)Math.Sqrt((double)(1 - dx * dx));
                    if ((player.RotationAngle < pi * 0.5f) && (player.RotationAngle > pi * -0.5f))  // 2 fall wenn hoch fliegen
                        dy *= -1.0f;
                    player.Velocity_X += dx * player.ActAcceleration();
                    player.Velocity_Y += dy * player.ActAcceleration();
                }
            }
            UniverseVar.TimeToRun = UniverseVar.TimeToRun.Subtract(new TimeSpan(DateTime.Now.Ticks - LastUpdateTime.Ticks));
            if (UniverseVar.TimeToRun.Hours*3600000.0f + UniverseVar.TimeToRun.Minutes*60000.0f+UniverseVar.TimeToRun.Seconds*1000.0f+ UniverseVar.TimeToRun.Milliseconds < 100)
            {// spielzeit ist abgelaufen, also rang ermitteln und spieler benachrichtigen
                EndGame();
            }
            MoveFieldElements(secondfraction);
            CollisionCheck();
            UpdateStations();
            ControlAIPlayers();
        }
        Locked = true;  // locken während kopieren falls was zugreifen will damit es konsistent ist
        UniverseCopy.MakeCopy(UniverseVar);
        Locked = false;
    }
    private void EndGame()
    {// beendet das spiel und informiert alle clients
        int i,j, temp;
        GameOver = true;    // wenn gameover gesetzt kann spiel nicht mehr unpausiert werden
        UniverseVar.GamePaused = true;
        List<int> Moneys = new List<int>();
        bool sorted;
        for (i = 0; i < UniverseVar.NumberofPlayers; i++)
        {// alle spieler durchgehen und geldbetrag extrahieren
            Moneys.Add(((Player)(UniverseVar.Elements[i])).Money);
        }
        do
        {// die gelder mit einfachem bubblesort sortieren
            sorted = true;
            for (i = 0; i < Moneys.Count - 1; i++)
            {
                if (Moneys[i] < Moneys[i + 1])
                {
                    temp = Moneys[i];
                    Moneys[i] = Moneys[i + 1];
                    Moneys[i + 1] = temp;
                    sorted = false;
                }
            }
        } while (false == sorted);
        for (i = 0; i < UniverseVar.NumberofPlayers; i++)
        {// alle spieler benachrichtigen
            for(j = 0; j < UniverseVar.NumberofPlayers; j++)
            {
                if(((Player)(UniverseVar.Elements[i])).Money == Moneys[j])
                    ((Player)(UniverseVar.Elements[i])).AddMessage(String.Format("Game Over. You reached Rank {0}",j+1));
            }
            ((Player)(UniverseVar.Elements[i])).Lost = true;
        }
        Console.WriteLine("Game Over");
    }
    private void UpdateStations()
    {// in den stationen die mengen anpassen
        if (DateTime.Now.Minute == LastUpdateMinute)    // erst wenn eine neue minute angebrochen ist die updates ausführen, solange gleich die fkt verlassen
            return;
        LastUpdateMinute = DateTime.Now.Minute; // zeit für nächstes mal merken
        Station station;
        for (int i = 0; i < UniverseVar.Elements.Count; i++)
        {
            if (6 == UniverseVar.Elements[i].Type)
            {// wenn element eine station ist die poduktion und verbrauch auf die vorräte anwenden
                station = (Station)UniverseVar.Elements[i];
                if(station.Supply >= UniverseVar.StationSupplyUsageRate)
                {// nur wenn eine station ausreichend supply hat tut sie erz produzieren (supply wird auf jeden fall verbraucht um platz zu schaffen für neues erz)
                    station.Supply -= UniverseVar.StationSupplyUsageRate;
                    if (station.Supply + station.Ore + UniverseVar.StationProductionRate <= station.CargoCapacity)
                    {// wenn voll nichts mehr produzieren, es werden nur ganze zyklen produziert
                        station.Ore += UniverseVar.StationProductionRate;
                    }
                }
            }
        }
    }
    private void CollisionCheck()
    {// die kollisionen prüfen und bewegungswinkel anpassen
        int i, j;
        float distx, disty,collisiondistsq = 128.0f*128.0f,radius = (float)UniverseVar.Radius,newspeed, oldspeed,olddistsq;
        FieldElement actelement;
        for (i = 0; i < UniverseVar.Elements.Count; i++)
        {// für jedes Element prüfen ob es sich aus spielfeld rausbewegt hat oder einem anderen zu nahe ist
            actelement = UniverseVar.Elements[i];
            olddistsq = actelement.Position_X * actelement.Position_X + actelement.Position_Y * actelement.Position_Y;
            if(i < UniverseVar.NumberofPlayers)
                ((Player)UniverseVar.Elements[i]).InsideDockingrange = -1;   // docking für raumschiffe zurücksetzen und in der funktion erneut prüfen
            if (olddistsq > radius * radius)
            {// checken ob der punkt im spielfeld-kreis liegt, wenn nicht neuen zufallsvektor bestimmen der wieder zurück führt (nächste position näher dran)
                do
                {
                    oldspeed = actelement.GetSpeed();
                    actelement.Velocity_X = ((float)RandomNumberGenerator.NextDouble() * (float)UniverseVar.MaxAsteroidVelocity * 2.0f) - UniverseVar.MaxAsteroidVelocity;// vektor zufällig erzeugen
                    actelement.Velocity_Y = ((float)RandomNumberGenerator.NextDouble() * (float)UniverseVar.MaxAsteroidVelocity * 2.0f) - UniverseVar.MaxAsteroidVelocity;
                    newspeed = (float)Math.Sqrt((double)(actelement.Velocity_X * actelement.Velocity_X + actelement.Velocity_Y * actelement.Velocity_Y));
                    actelement.Velocity_X /= newspeed;  // vektor normieren
                    actelement.Velocity_Y /= newspeed;
                    actelement.Velocity_X *= oldspeed;  // mit alter geschw multiplizieren damit geschw erhalten bleibt und nur die richtung geändert wird
                    actelement.Velocity_Y *= oldspeed;
                } while (olddistsq > (actelement.Position_X - actelement.Velocity_X) * (actelement.Position_X - actelement.Velocity_X) + (actelement.Position_Y - actelement.Velocity_Y) * (actelement.Position_Y - actelement.Velocity_Y));// prüfen ob nächste position (mit neuem vektor) näher am zentrum dran liegt als die alte
            }
            for (j = i + 1; j < UniverseVar.Elements.Count; j++)
            {
                distx = UniverseVar.Elements[i].Position_X - UniverseVar.Elements[j].Position_X;
                disty = UniverseVar.Elements[i].Position_Y - UniverseVar.Elements[j].Position_Y;
                if ((distx * distx + disty * disty) < collisiondistsq)
                {// kollision erzeugen (sound spielen, schaden berechnen (wenn schiff involviert), vektoren ändern)
                    if (6 > UniverseVar.Elements[i].Type)
                    {// erstes element ist ein asteroid
                        if (6 > UniverseVar.Elements[j].Type)
                            Collide(i, j);  // asteroid - asteroid
                        if (6 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // asteroid - station
/*                        if (7 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // asteroid - frachter*/
                        if (8 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // asteroid - raumschiff
                    }
                    if (6 == UniverseVar.Elements[i].Type)
                    {// erstes element ist eine station
                        if (6 > UniverseVar.Elements[j].Type)
                            Collide(i, j);  // station - asteroid
                        if (6 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // station - station
/*                        if (7 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // station - frachter*/
                        if (8 == UniverseVar.Elements[j].Type)
                        {
                            ((Player)UniverseVar.Elements[j]).InsideDockingrange = i;  // station - raumschiff
                            if ((j >= NumberofExpectedClients) && (i == ((Player)UniverseVar.Elements[j]).TargetIndex))
                            {// ein ki-spieler
                                Docking(j); // den spieler an der station andocken sobald er in reichweite ist
                            }
                        }
                    }
                    if (7 == UniverseVar.Elements[i].Type)
                    {// erstes element ist ein frachter
/*                         if (6 > UniverseVar.Elements[j].Type)
                            Collide(i, j);  // frachter - asteroid
                        if (6 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // frachter - station
                        if (7 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // frachter - frachter*/
                        if (8 == UniverseVar.Elements[j].Type)
                        {
                            ((Player)UniverseVar.Elements[j]).InsideDockingrange = i;  // frachter - raumschiff
                            if ((j >= NumberofExpectedClients) && (i == ((Player)UniverseVar.Elements[j]).TargetIndex))
                            {// ein ki-spieler
                                Docking(j); // den spieler an dem frachter andocken sobald er in reichweite ist
                            }
                        }
                    }
                    if (8 == UniverseVar.Elements[i].Type)
                    {// erstes element ist ein spieler (raumschiff)
                        if (6 > UniverseVar.Elements[j].Type)
                            Collide(i, j);  // raumschiff - asteroid
                        if (6 == UniverseVar.Elements[j].Type)
                        {
                            ((Player)UniverseVar.Elements[i]).InsideDockingrange = j;  // raumschiff - station
                            if ((i >= NumberofExpectedClients) && (j == ((Player)UniverseVar.Elements[i]).TargetIndex))
                            {// ein ki-spieler
                                Docking(i); // den spieler an der station andocken sobald er in reichweite ist
                            }
                        }
                        if (7 == UniverseVar.Elements[j].Type)
                        {
                            ((Player)UniverseVar.Elements[i]).InsideDockingrange = j;  // raumschiff - frachter
                            if ((i >= NumberofExpectedClients) && (j == ((Player)UniverseVar.Elements[i]).TargetIndex))
                            {// ein ki-spieler
                                Docking(i); // den spieler an dem frachter andocken sobald er in reichweite ist
                            }
                        }
                        if (8 == UniverseVar.Elements[j].Type)
                            Collide(i, j);  // raumschiff - raumschiff
                    }
                }
            }
        }
    }
    protected float GetRotation(float TargetvectorX, float TargetvectorY)
    {// berechnet die rotation aus dem upvektor und den zielvektor
        float rotation = (float)Math.Acos((double)TargetvectorY / Math.Sqrt(TargetvectorX * TargetvectorX + TargetvectorY * TargetvectorY));   // rotationswinkel aus skalarprodukt der vektoren (upvektor) berechnen
        if (TargetvectorX < 0.0f)
            return rotation;
        else
            return rotation * -1.0f;
    }
    private void MoveFieldElements(float secondfraction)
    {// alle elemente werden entsprechend ihres bewegungsvektors bewegt
        float pi = (float)Math.PI;
        for (int i = 0; i < UniverseVar.Elements.Count; i++)
        {
            UniverseVar.Elements[i].Position_X += UniverseVar.Elements[i].Velocity_X * secondfraction;
            UniverseVar.Elements[i].Position_Y += UniverseVar.Elements[i].Velocity_Y * secondfraction;
            if (UniverseVar.Elements[i].Type < 7)   // stationen und asteroiden rotieren, frachter rotieren nicht
                UniverseVar.Elements[i].RotationAngle += UniverseVar.Elements[i].RotationSpeed * secondfraction;
            if(8 == UniverseVar.Elements[i].Type)
            {// schiffe werden durch steuerung und ki rotiert (targetrotation)
                Player player = (Player)UniverseVar.Elements[i];
                if (player.RotationAngle < 0.0f)
                {// schiff zeigt nach links
                    if ((player.RotationAngle + pi > player.Targetrotation) && (player.RotationAngle < player.Targetrotation)) // nach rechts drehen
                        player.RotationAngle += player.ActAgility() * secondfraction;
                    else
                        player.RotationAngle -= player.ActAgility() * secondfraction;
                }
                else
                {// schiff zeigt nach rechts
                    if ((player.RotationAngle - pi < player.Targetrotation) && (player.RotationAngle > player.Targetrotation)) // nach links drehen
                        player.RotationAngle -= player.ActAgility() * secondfraction;
                    else
                        player.RotationAngle += player.ActAgility() * secondfraction;
                }
                if (player.RotationAngle > pi)  // zu große und zu kleine werte anpassen (-pi < winkel < pi)
                    player.RotationAngle -= pi * 2.0f;
                if (player.RotationAngle < pi * -1.0f)
                    player.RotationAngle += pi * 2.0f;
            }
            if ((8 == UniverseVar.Elements[i].Type) && (((Player)(UniverseVar.Elements[i])).Dockingindex >= 0)) // das schiff ist wo gedockt also die schiffsposition auf die stations-/frachterposition beziehene
            {
                UniverseVar.Elements[i].Position_X = UniverseVar.Elements[((Player)(UniverseVar.Elements[i])).Dockingindex].Position_X + ((Player)(UniverseVar.Elements[i])).DockedPos_X;
                UniverseVar.Elements[i].Position_Y = UniverseVar.Elements[((Player)(UniverseVar.Elements[i])).Dockingindex].Position_Y + ((Player)(UniverseVar.Elements[i])).DockedPos_Y;
            }
        }
    }
    private void Collide(int index1, int index2)
    {// kollidiert die 2 übergebenen / indizierten elemente
        float diffx, diffy,midx,midy,newspeed,oldspeed,velx,vely;
        int damage = 0;
        diffx = UniverseVar.Elements[index1].Position_X - UniverseVar.Elements[index2].Position_X;
        diffy = UniverseVar.Elements[index1].Position_Y - UniverseVar.Elements[index2].Position_Y;
        midx = UniverseVar.Elements[index2].Position_X + diffx * 0.5f;        // punkt dazwischen berechnen
        midy = UniverseVar.Elements[index2].Position_Y + diffy * 0.5f;
        // den abprallwinkel von objekt 1 bestimmen
        oldspeed = UniverseVar.Elements[index1].GetSpeed();
        UniverseVar.Elements[index1].Velocity_X = UniverseVar.Elements[index1].Position_X - midx;        // von dem punkt aus die neue beschl in richtung des zentrums bestimmen
        UniverseVar.Elements[index1].Velocity_Y = UniverseVar.Elements[index1].Position_Y - midy;
        newspeed = (float)Math.Sqrt((double)(UniverseVar.Elements[index1].Velocity_X * UniverseVar.Elements[index1].Velocity_X + UniverseVar.Elements[index1].Velocity_Y * UniverseVar.Elements[index1].Velocity_Y));
        UniverseVar.Elements[index1].Velocity_X /= newspeed;
        UniverseVar.Elements[index1].Velocity_Y /= newspeed;
        UniverseVar.Elements[index1].Velocity_X *= oldspeed;        // normieren mit alter geschw.
        UniverseVar.Elements[index1].Velocity_Y *= oldspeed;
        // den abprallwinkel von objekt 2 bestimmen
        oldspeed = UniverseVar.Elements[index2].GetSpeed();
        UniverseVar.Elements[index2].Velocity_X = UniverseVar.Elements[index2].Position_X - midx;        // von dem punkt aus die neue beschl in richtung des zentrums bestimmen
        UniverseVar.Elements[index2].Velocity_Y = UniverseVar.Elements[index2].Position_Y - midy;
        newspeed = (float)Math.Sqrt((double)(UniverseVar.Elements[index2].Velocity_X * UniverseVar.Elements[index2].Velocity_X + UniverseVar.Elements[index2].Velocity_Y * UniverseVar.Elements[index2].Velocity_Y));
        UniverseVar.Elements[index2].Velocity_X /= newspeed;
        UniverseVar.Elements[index2].Velocity_Y /= newspeed;
        UniverseVar.Elements[index2].Velocity_X *= oldspeed;        // normieren mit alter geschw.
        UniverseVar.Elements[index2].Velocity_Y *= oldspeed;

        Player player;
        velx = UniverseVar.Elements[index1].Velocity_X - UniverseVar.Elements[index2].Velocity_X;
        vely = UniverseVar.Elements[index1].Velocity_Y - UniverseVar.Elements[index2].Velocity_Y;
        damage = (int)Math.Sqrt(velx*velx+vely*vely);   // beide elemente bekommen denselben damage ab sofern sie verwundbar sind (keine asteroiden usw)
        if ((8 == UniverseVar.Elements[index1].Type) && (-1 == ((Player)(UniverseVar.Elements[index1])).Dockingindex) && (false == ((Player)(UniverseVar.Elements[index1])).Lost))  // nur wenn nicht gedockt schaden anwenden
        {// ein spieler
            player = ((Player)(UniverseVar.Elements[index1]));
            player.ActHitpoints -= damage;
            if (0 > damage)
                damage = 0;
            player.AddMessage(String.Format("Collision. {0} Damage taken. {1} / {2} Hitpoints remaining.", damage, player.ActHitpoints, player.MaxHitpoints()));
            if (0 > player.ActHitpoints)
            {// der maximale schaden wurde erreicht, spieler zerstören und benachrichtigen
                player.AddMessage("Ship Destroyed. Game Over.");
                player.Lost = true;
                player.Velocity_X = player.Velocity_Y = 0.0f;
            }
        }
        if ((8 == UniverseVar.Elements[index2].Type) && (-1 == ((Player)(UniverseVar.Elements[index2])).Dockingindex) && (false == ((Player)(UniverseVar.Elements[index2])).Lost))  // nur wenn nicht gedockt schaden anwenden
        {
            player = ((Player)(UniverseVar.Elements[index2]));
            player.ActHitpoints -= damage;
            if (0 > damage)
                damage = 0;
            player.AddMessage(String.Format("Collision. {0} Damage taken. {1} / {2} Hitpoints remaining.", damage, player.ActHitpoints, player.MaxHitpoints()));
            if (0 > player.ActHitpoints)
            {// der maximale schaden wurde erreicht, spieler zerstören und benachrichtigen
                player.AddMessage("Ship Destroyed. Game Over.");
                player.Lost = true;
                player.Velocity_X = player.Velocity_Y = 0.0f;
            }
        }
    }
    private void ControlAIPlayers()
    {//list der players, die die keine spieler sind werden ki gesteuert (funktion) dabei sind die ersten bis anz menschl spieler alle menschen und der rest ist ki,
        Player player, tempplayer;
        Station tempstation;
        Freighter tempfreighter;
        int i, j, maxindex = 0, level,price,oldmoney,amount,totalprice, maxlevel=0, minutes = UniverseVar.TimeToRun.Hours * 60 + UniverseVar.TimeToRun.Minutes;
        float actscore, maxscore, playerx,playery, objx,objy,targetdist,moneyreserve,eta,targetx,targety, maxvel = 50.0f, velorot,maxtraveltime=UniverseVar.Radius*2.0f/maxvel;
        DateTime nowtime = DateTime.Now;
        for (i = NumberofExpectedClients; i < UniverseVar.NumberofPlayers; i++)
        {// große for schleife durch alle ki players
            player = (Player)UniverseVar.Elements[i];
            if(player.LastActionTime.AddSeconds(0.6) > nowtime) // alle 0.6 sekunden updaten damit alle 2 updates die triebwerke gezündet werden können
                continue;   // der spieler wurde erst vor kurzem upgedated
            player.LastActionTime = nowtime;    // zeit für nächstes mal merken
            if (player.Dockingindex > -1)
            {// spieler ist gedockt, handeln, resupply, upgrades kaufen, abdocken,
                Trade(i);   // alles handeln mit oberster priorität (da einzige geldquelle)
                Refuel(i);  // resupply
                Repair(i);
                moneyreserve = player.Money - 1.0f * player.MaxCargoCapacity() * (float)Math.Max(UniverseVar.SupplyMinPrice, UniverseVar.OreMaxPrice); // genug geld in reserve halten um cargo 2 mal voll zu kaufen bei schlechtesten bedingungen
                if(minutes > 10)    // mehr als 10 minuten verbleiben also bis lvl 1 upgrades kaufen (die könnten sich rentieren), der kauf wird natürlich noch durch genügend geld(reserve) limitiert
                    maxlevel = 1;
                if (minutes > 30)    // mehr als 30 minuten verbleiben also bis lvl 2 upgrades kaufen (die könnten sich rentieren), der kauf wird natürlich noch durch genügend geld(reserve) limitiert
                    maxlevel = 2;
                if (minutes > 70)    // mehr als 70 minuten verbleiben also bis lvl 3 upgrades kaufen (die könnten sich rentieren), der kauf wird natürlich noch durch genügend geld(reserve) limitiert
                    maxlevel = 3;
                if (minutes > 150)    // mehr als 150 minuten verbleiben also bis lvl 4 upgrades kaufen (die könnten sich rentieren), der kauf wird natürlich noch durch genügend geld(reserve) limitiert
                    maxlevel = 4;
                if (minutes > 310)    // mehr als 310 minuten verbleiben also bis lvl 5 upgrades kaufen (die könnten sich rentieren), der kauf wird natürlich noch durch genügend geld(reserve) limitiert
                    maxlevel = 5;
                player.GetCargoUpgrData(out level, out price);
                if ((moneyreserve > price) && (level <= maxlevel))
                {// cargoupgrade kaufen
                    player.BuyCargoUpgrade();
                    moneyreserve -= price;
                }
                player.GetFuelUpgrData(out level, out price);
                if ((moneyreserve > price) && (level <= maxlevel))
                {// fuelupgrade kaufen
                    player.BuyFuelUpgrade();
                    moneyreserve -= price;
                }
                player.GetHitpUpgrData(out level, out price);
                if ((moneyreserve > price) && (level <= maxlevel))
                {// panzerungsupgrade kaufen
                    player.BuyHitpointUpgrade();
                    moneyreserve -= price;
                }
                player.GetThrustUpgrData(out level, out price);
                if ((moneyreserve > price) && (level <= maxlevel))
                {// antriebsupgrade kaufen
                    player.BuyAccelerationUpgrade();
                    moneyreserve -= price;
                }
                player.GetAgilUpgrData(out level, out price);
                if ((moneyreserve > price) && (level <= maxlevel))
                {
                    player.BuyAgilityUpgrade();
                    moneyreserve -= price;
                }
/*                player.GetBrakeUpgrData(out level, out price);
                if ((moneyreserve > price) && (level <= maxlevel))
                {// bremsupgrade kaufen, muss nicht gekauft werden weil die ki voll bremsen kann (ohne treibstoffverbrauch) für fairness
                    player.BuyBrakeUpgrade();
                    moneyreserve -= price;
                }*/
                Docking(i); // abdocken
                player.TargetIndex = -1;    // beim nächsten mal neues ziel suchen
            }
            else
            {// spieler ist im raum also alle verfügbaren dockingmöglichkeiten auf eignung prüfen (score) und anfliegen
                playerx = player.Position_X;
                playery = player.Position_Y;
                if ((-1 == player.TargetIndex) || (true == UniverseVar.Elements[player.TargetIndex].Changed))
                {// kein aktuelles ziel gewählt oder das ausgewählte verändert wurde (handel) also eines auswählen und dabei bleiben
                    maxscore = -100000.0f;    // beim score berechnen auch bestimmen wie viel ich verdienen kann durch kauf/verkauf, und weg/treibstoff mit rein und zeit bis dahin, das einfach für jede statione machen ohne zu berückscihtigen was danach kommt,
                    for (j = UniverseVar.NumberofPlayers; j < UniverseVar.Elements.Count; j++)
                    {// alle stationen + frachter durchgehen und mit entfernung dahin sowie den werten einen score ausrechnen,
                        if ((6 == UniverseVar.Elements[j].Type) || (7 == UniverseVar.Elements[j].Type))
                        {
                            objx = UniverseVar.Elements[j].Position_X;
                            objy = UniverseVar.Elements[j].Position_Y;
                            targetdist = (float)Math.Sqrt((double)((playerx - objx) * (playerx - objx) + (playery - objy) * (playery - objy))); // länge des vektors zum ziel (entfernung in pixel)
                            if ((targetdist < 128.0f) && ((player.ActFuel / player.MaxFuel() < 0.3f) || (player.ActHitpoints / player.MaxHitpoints() < 0.3f)))
                            {// spieler ist in bedrängnis, also wenn zufällig was in reichweite unbedingt docken (auch wenn man nur noch driftet oder die station zu einem kommt)
                                player.Dockingindex = j;    // schnell mal andocken zur versorgung
                                Trade(i);   // alles handeln mit oberster priorität (da einzige geldquelle)
                                Refuel(i);  // resupply
                                Repair(i);
                                if (player.ActFuel / player.MaxFuel() < 0.3f)   // wenn alles nichts genutzt hat spieler unter die arme greifen und "cheaten" damit er wenigstens noch mitmachen kann
                                    player.ActFuel = player.MaxFuel() * 0.5f;
                                if(player.ActHitpoints / player.MaxHitpoints() < 0.3f)
                                    player.ActHitpoints = player.MaxHitpoints() / 2;
                                player.Dockingindex = -1;
                            }
                            eta = targetdist / maxvel;// +(player.RotationAngle - player.Targetrotation) / player.ActAgility(); // die vorraussichtliche flugdauer bei höchstgeschw. in sekunden dahin berechnen +( beschleunigungszeit +) rotationszeit
                            tempplayer = new Player(player);    // temporäre kopie fürs handeln erzeugen
                            oldmoney = tempplayer.Money;
                            if (6 == UniverseVar.Elements[j].Type)
                            {    // auch die ladung einbeziehen (kauf/verkauf) also temptrade machen (simulation)
                                tempstation = new Station((Station)UniverseVar.Elements[j]);    // temporäre kopie fürs handeln erzeugen
                                tempplayer.Trade(tempstation, out amount, out totalprice);
                            }
                            else if (7 == UniverseVar.Elements[j].Type)
                            {
                                tempfreighter = new Freighter((Freighter)UniverseVar.Elements[j]);    // temporäre kopie fürs handeln erzeugen
                                tempplayer.Trade(tempfreighter, out amount, out totalprice);
                            }
                            actscore = ((float)(Math.Abs(tempplayer.Money - oldmoney))) / eta; // score ist verdientes geld pro zurückgelegtem weg
                            if (actscore > maxscore)
                            {// diese station/frachter ist besser geeignet als alle bisher also merken
                                maxscore = actscore;
                                maxindex = j;
                            }
                        }
                    }
                    player.TargetIndex = maxindex;  // falls in reichweite daran andocken
                }
                objx = UniverseVar.Elements[player.TargetIndex].Position_X;
                objy = UniverseVar.Elements[player.TargetIndex].Position_Y;
                targetdist = (float)Math.Sqrt((double)((playerx - objx) * (playerx - objx) + (playery - objy) * (playery - objy))); // länge des vektors zum ziel (entfernung in pixel)
                eta = targetdist / maxvel; // die vorraussichtliche flugdauer bei höchstgeschw. in sekunden dahin berechnen für vorhaltepunkt
                targetx = ((playerx) - (objx + eta * UniverseVar.Elements[player.TargetIndex].Velocity_X)) / targetdist * maxvel;    // einheitsvektor in zielrichtung bestimmen und mit maximalgeschw multiplizieren
                targety = ((playery) - (objy + eta * UniverseVar.Elements[player.TargetIndex].Velocity_Y)) / targetdist * maxvel;
                player.Targetrotation = GetRotation(targetx,targety);   // auf zukünftige zielposition ausrichten
                velorot = GetRotation(-1.0f * player.Velocity_X, -1.0f * player.Velocity_Y);    // in diese drehrichtung bewegt sich schiff
                if ((targetdist > player.LastTargetDist) && (targetdist < 400.0f))  // falls letzte position näher dran war die bremse versuchen (vorbeigeflogen)
                    Brake(i);
                player.LastTargetDist = targetdist; // merken für nächstes frame
                if ((Math.Abs(Math.Abs(player.RotationAngle) - Math.Abs(player.Targetrotation)) < 0.4f) && ((Math.Abs(Math.Abs(player.RotationAngle) - Math.Abs(velorot))) > 0.8f) || (player.GetSpeed() < maxvel) || ((targetdist > player.LastTargetDist) && (targetdist > 400.0f))) // sobald schiff halbswegs in die richtung zeigt beschleunigen falls zu langsam oder in falsche richtung bewegt
                    FireThrusters(i);
            }
        }
        for (i = 0; i < UniverseVar.Elements.Count; i++)
        {// da alle ki-spieler die änderungen registriert haben die variable zurücksetzen
            UniverseVar.Elements[i].Changed = false;
        }
    }
}