﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

[Serializable]

public class Player : FieldElement
{// repräsentiert einen menschlichen oder ki-spieler
    public List<String> Messages;   // nimmt textnachrichten auf um spieler zu informieren
    public DateTime LastActionTime;    // letzte abfrage von diesem client oder letzte aktualisierung der ki
    public bool Lost;  // Lost gibt an das verbindung weg ist oder spieler zerstört wurde
    public bool ThrustersFiring;    // die triebwerke des schiffes sind aktiv
    public DateTime ThrusterTime;   // speichert wann das letzte mal die Triebwerke gezündet wurden für brenndauer (zb 1 sec)
    public int InsideDockingrange;   // schiff befindet sich in der entfernung zu station oder frachter mit dem Index um andocken zu können
    public int Dockingindex;    // index der station/frachter wo man angedockt ist (für kaufen und für position da sich stationen bewegen)
    public float DockedPos_X, DockedPos_Y;  // die position relativ zu station oder frachter wo man angedockt hat
    public int LoadedOre;   // die menge an geladenem erz für die Frachter
    public int LoadedSupply;    // die menge an geladenem Life Supply für die Stationen
    public int UpgradeCostFactor;    // um diesen faktor kostet das nächste upgrade mehr als das vorherige
    public int UpgradeCostBase;  // so viel kostet das erste upgrade
    public int MaxUpgradeLevel;  // so viele Level können maximal gekauft werden
    public int Money;   // der geldbetrag den der Spieler hat
    public int ActHitpoints;    // wie viel noch übrig ist
    public int MaxMessages; // so viele nachrichten speichert der spieler
    public float ActFuel;    // die aktuelle Menge treibstoff
    public int TargetIndex; // das ziel (index) für die KI (evtl später noch das für spieler aber nutzlos solange keine interaktion (abschießen))
    public float LastTargetDist;    //die entfernung während dem letzten update um vorbeiflug zu registrieren
    public float Targetrotation;
    private float FactorCargohold, FactorHitpoints, FactorFuel, FactorAgility, FactorAcceleration, FactorBrake; // der aktuelle durch upgrades beeinflusste faktor (start bei 1, änderung jeweils um einen bestimmten wert pro stufe)
    private int LevelCargohold, LevelHitpoints, LevelFuel, LevelAgility, LevelAcceleration, LevelBrake;  // der aktuelle level (start bei 0 also unmodifiziert)
    private int BaseCargoCapacity, BaseHitpoints; // der basiswert von dem die upgrades ausgehen und aus dem der aktuelle wert prozentual berechnet wird
    private float BaseAgility, BaseAcceleration, BaseBrake, BaseFuel;
    public bool ArtificIntelligence;

    public Player(float posx, float posy, float velx, float vely, float rotang, float rotspd, int type, int count, bool dockable, int fuel, int hitpoints, int cargo, int upgcostfact, int upgcostbase, int maxupglvl, int maxmess, int money): base(posx, posy, velx, vely, rotang, rotspd, type, count, dockable)
    {
        Lost = ThrustersFiring = ArtificIntelligence = false;
        RotationSpeed = Targetrotation = 0.0f;   // die rotation von spielern wird über die agility geregelt
        BaseFuel = ActFuel = fuel;
        Dockingindex = InsideDockingrange = TargetIndex  = - 1;  // ungedockt, beim docken auf das entsprechende element setzen
        BaseHitpoints = ActHitpoints = hitpoints;
        BaseCargoCapacity = cargo;
        FactorCargohold = FactorHitpoints = FactorFuel = FactorAgility = FactorAcceleration = FactorBrake = 1.0f;
        LevelCargohold = LevelHitpoints = LevelFuel = LevelAgility = LevelAcceleration = LevelBrake = 0;    // spiel beginnt ohne upgrades
        Messages = new List<string>();
        BaseAgility = (float)Math.PI * 2.0f / 8.0f; // in 8 sekunden eine volldrehung machen ohne upgrades
        BaseAcceleration = 3.0f;    // mit einem pixel pro sekunde beschleunigen ohne upgrades,
        BaseBrake = 1.0f;    // ohne upgrades bremst es überhaupt nicht
        DockedPos_X = DockedPos_Y = 0.0f;
        Money = money; // Startgeld für einkäufe und reparaturen
        UpgradeCostFactor = upgcostfact;
        UpgradeCostBase = upgcostbase;
        MaxUpgradeLevel = maxupglvl;
        MaxMessages = maxmess;
        LastTargetDist = 1000000.0f;    // auf großen wert setzen damit es beim nächsten check kleiner ist (annäherung)
    }
    public Player(Player orig): base(orig.Position_X, orig.Position_Y, orig.Velocity_X, orig.Velocity_Y, orig.RotationAngle, orig.RotationSpeed, orig.Type, orig.Count, orig.Dockable)
    {// eine teilweise kopie fürs handeln erzeugen also nur die relevanten elemente kopieren
        this.LoadedOre = orig.LoadedOre;
        this.LoadedSupply = orig.LoadedSupply;
        this.Money = orig.Money;
        this.FactorCargohold = orig.FactorCargohold;
        this.LevelCargohold = orig.LevelCargohold;
        this.BaseCargoCapacity = orig.BaseCargoCapacity;
        this.Dockingindex = orig.Dockingindex;
        this.Messages = new List<string>();
    }
    public void AddMessage(String text)
    {// fügt die message in die Liste ein (und entfernt evtl vorhandene falls zu viele)
        if (false == Lost)
        {// nur wenn spieler noch nicht verloren hat messages einfügen
            Messages.Add(text);
            if (Messages.Count > MaxMessages)
                Messages.RemoveAt(0);
        }
    }
    public void GetRepairData(out int amount, out int totalprice, int repairprice)
    {// wenn schiff repariert werden soll
        amount = Math.Min(MaxHitpoints() - ActHitpoints, Money / repairprice);  // bis alles repariert ist, was ich mir leisten kann
        totalprice = amount * repairprice;
    }
    public void GetRefuelData(out int amount, out int totalprice, int repairprice)
    {
        amount = Math.Min((int)(MaxFuel() - ActFuel), Money / repairprice); // wie viel in den Tank passt, was ich mir leisten kann
        totalprice = amount * repairprice;
    }
    public void GetAgilUpgrData(out int amount, out int totalprice)
    {
        amount = LevelAgility; // den aktuellen level zurückgeben
        totalprice = UpgradeCost(amount +1);
    }
    public void GetBrakeUpgrData(out int amount, out int totalprice)
    {
        amount = LevelBrake; // den aktuellen level zurückgeben
        totalprice = UpgradeCost(amount + 1);
    }
    public void GetHitpUpgrData(out int amount, out int totalprice)
    {
        amount = LevelHitpoints; // den aktuellen level zurückgeben
        totalprice = UpgradeCost(amount + 1);
    }
    public void GetCargoUpgrData(out int amount, out int totalprice)
    {
        amount = LevelCargohold; // den aktuellen level zurückgeben
        totalprice = UpgradeCost(amount + 1);
    }
    public void GetFuelUpgrData(out int amount, out int totalprice)
    {
        amount = LevelFuel; // den aktuellen level zurückgeben
        totalprice = UpgradeCost(amount + 1);
    }
    public void GetThrustUpgrData(out int amount, out int totalprice)
    {
        amount = LevelAcceleration; // den aktuellen level zurückgeben
        totalprice = UpgradeCost(amount + 1);
    }
    public void GetSellOreData(out int amount, out int totalprice, Freighter freighter)
    {// wenn erz auf frachter verkauft werden soll
        amount = LoadedOre;
        totalprice = amount * freighter.GetOrePrice();
    }
    public void GetBuySupplyData(out int amount, out int totalprice, Freighter freighter)
    {// wenn erz auf frachter verkauft werden soll, ACHTUNG: Funktion muss immer NACH GetSellOreData mit deren amount aufgerufen werden
        amount = Math.Min(MaxCargoCapacity() - LoadedSupply, Money / freighter.GetSupplyPrice()); // was reinpasst, was ich mir leisten kann
        totalprice = amount * freighter.GetSupplyPrice();
    }
    public void GetBuyOreData(out int amount, out int totalprice, Station station)
    {// wenn erz auf station gekauft werden soll, ACHTUNG: Funktion muss immer NACH GetSellSupplyData mit deren amount aufgerufen werden
        amount = Math.Min(MaxCargoCapacity() - LoadedOre - LoadedSupply, Math.Min(Money  / station.GetOrePrice(), station.Ore)); // was reinpasst, was ich mir leisten kann, was auf station vorhanden ist
        totalprice = amount * station.GetOrePrice();
    }
    public void GetSellSupplyData(out int amount, out int totalprice, Station station)
    {// wenn supply auf station verkauft werden soll
        amount = Math.Min(LoadedSupply,  station.CargoCapacity - station.Supply); // was ich geladen habe, was in station reinpasst (nachdem alles erz gekauft wurde), was wenn ich nur teil des erzes kaufen kann wegen platz oder kosten?
        totalprice = amount * station.GetSupplyPrice();
    }
    public void Trade(FieldElement dockedat, out int amount, out int totalprice)
    {// handeln (eins verkaufen das andere kaufen
//#        int amount, totalprice;
        if (6 == dockedat.Type)
        {// station
            Station station = (Station)dockedat;
            GetSellSupplyData(out amount, out totalprice, station);
            LoadedSupply -= amount;
            Money += totalprice;
            station.Supply += amount;
            AddMessage(String.Format("Selled -{0} units Supply for +{1} money", amount, totalprice));
            GetBuyOreData(out amount, out totalprice, station);
            LoadedOre += amount;
            Money -= totalprice;
            station.Ore -= amount;
            AddMessage(String.Format("Buyed +{0} units Ore for -{1} money",amount, totalprice));
            station.Changed = true;   // merken dass es verändert wurde für den nächsten ki-durchlauf
        }
        else
        {// frachter
            Freighter freighter = (Freighter)dockedat;
            GetSellOreData(out amount, out totalprice, freighter);
            LoadedOre -= amount;
            Money += totalprice;
            AddMessage(String.Format("Selled -{0} units Ore for +{1} money", amount, totalprice));
            GetBuySupplyData(out amount, out totalprice, freighter);
            LoadedSupply += amount;
            Money -= totalprice;
            AddMessage(String.Format("Buyed +{0} units Supply for -{1} money", amount, totalprice));
            freighter.Changed = true;   // merken dass es verändert wurde für den nächsten ki-durchlauf
        }
    }
    public void Repair(FieldElement dockedat, int repairprice)
    {// repariert die schiffspanzerung
        int amount, totalprice;
        GetRepairData(out amount, out totalprice, repairprice);
        ActHitpoints += amount;
        Money -= totalprice;
        AddMessage(String.Format("Repaired +{0} units Damage for -{1} money", amount, totalprice));
    }
    public void Refuel(FieldElement dockedat, int repairprice)
    {// füllt den treibstofftank auf
        int amount, totalprice;
        GetRefuelData(out amount, out totalprice, repairprice);
        ActFuel += amount;
        Money -= totalprice;
        AddMessage(String.Format("Refueld +{0} units Fuel for -{1} money", amount, totalprice));
    }
    private int UpgradeCost(int nextlevel)
    {// berechnet was das nächste level eines upgrades kosten würde
        return UpgradeCostBase * (int)Math.Pow((double)UpgradeCostFactor, (double)(nextlevel - 1));
    }
    public void BuyCargoUpgrade()
    {// die prozentzahl pro stufe aus den maxstufen ausrechnen damit bei maxstufe doppelt so viel
        if (MaxUpgradeLevel == LevelCargohold)
        {// maximale Zahl Stufen erreicht
            AddMessage(String.Format("You have already purchased {0} Cargo Upgrades.",MaxUpgradeLevel));
            return;
        }
        int NextLevelCost = UpgradeCost(LevelCargohold+1);
        if (Money < NextLevelCost)
        {// zu wenig geld
            AddMessage(String.Format("You cannot afford Level {0} of Cargo Upgrade (only {1} of {2} Money).", LevelCargohold + 1, Money, NextLevelCost));
            return;
        }// alles ok, upgrade kaufen
        FactorCargohold = 1.0f + (1.0f * (float)++LevelCargohold / (float)MaxUpgradeLevel);    // beim maximalen upgrade level hat man doppelt so viel cargospace
        Money -= NextLevelCost;
        AddMessage(String.Format("Cargo Upgrade Level {0} purchased for -{1} Money.", LevelCargohold, NextLevelCost));
    }
    public void BuyHitpointUpgrade()
    {// die prozentzahl pro stufe aus den maxstufen ausrechnen damit bei maxstufe doppelt so viel
        if (MaxUpgradeLevel == LevelHitpoints)
        {// maximale Zahl Stufen erreicht
            AddMessage(String.Format("You have already purchased {0} Armor Upgrades.", MaxUpgradeLevel));
            return;
        }
        int NextLevelCost = UpgradeCost(LevelHitpoints + 1);
        if (Money < NextLevelCost)
        {// zu wenig geld
            AddMessage(String.Format("You cannot afford Level {0} of Armor Upgrade (only {1} of {2} Money).", LevelHitpoints + 1, Money, NextLevelCost));
            return;
        }// alles ok, upgrade kaufen
        int templosthp = MaxHitpoints() - ActHitpoints;
        FactorHitpoints = 1.0f + (1.0f * (float)++LevelHitpoints / (float)MaxUpgradeLevel);    // beim maximalen upgrade level hat man doppelt so viele hitpoints
        ActHitpoints = MaxHitpoints() - templosthp; // die verlorenen Hitpoints bleiben konstant (durch das upgrade zugefügte panzerung wird aufgefüllt)
        Money -= NextLevelCost;
        AddMessage(String.Format("Armor Upgrade Level {0} purchased for -{1} Money.", LevelHitpoints, NextLevelCost));
    }
    public void BuyFuelUpgrade()
    {// die prozentzahl pro stufe aus den maxstufen ausrechnen damit bei maxstufe doppelt so viel
        if (MaxUpgradeLevel == LevelFuel)
        {// maximale Zahl Stufen erreicht
            AddMessage(String.Format("You have already purchased {0} Fuel Upgrades.", MaxUpgradeLevel));
            return;
        }
        int NextLevelCost = UpgradeCost(LevelFuel + 1);
        if (Money < NextLevelCost)
        {// zu wenig geld
            AddMessage(String.Format("You cannot afford Level {0} of Fuel Upgrade (only {1} of {2} Money).", LevelFuel + 1, Money, NextLevelCost));
            return;
        }// alles ok, upgrade kaufen
        float templostfuel = MaxFuel() - ActFuel;
        FactorFuel = 1.0f + (1.0f * (float)++LevelFuel / (float)MaxUpgradeLevel);    // beim maximalen upgrade level hat man doppelt so viel tank
        ActFuel = MaxFuel() - templostfuel; // der verbrauchte treibstoff bleibt konstant (durch das upgrade zugefügter tank wird aufgefüllt)
        Money -= NextLevelCost;
        AddMessage(String.Format("Fuel Upgrade Level {0} purchased for -{1} Money.", LevelFuel, NextLevelCost));
    }
    public void BuyAgilityUpgrade()
    {// die prozentzahl pro stufe aus den maxstufen ausrechnen damit bei maxstufe halb so wendig wie zu beginn
        if (MaxUpgradeLevel == LevelAgility)
        {// maximale Zahl Stufen erreicht
            AddMessage(String.Format("You have already purchased {0} Agility Upgrades.", MaxUpgradeLevel));
            return;
        }
        int NextLevelCost = UpgradeCost(LevelAgility + 1);
        if (Money < NextLevelCost)
        {// zu wenig geld
            AddMessage(String.Format("You cannot afford Level {0} of Agility Upgrade (only {1} of {2} Money).", LevelAgility + 1, Money, NextLevelCost));
            return;
        }// alles ok, upgrade kaufen
        FactorAgility = 1.0f + (1.0f * (float)++LevelAgility / (float)MaxUpgradeLevel);    // beim maximalen upgrade level dreht man doppelt so schnell
        Money -= NextLevelCost;
        AddMessage(String.Format("Agility Upgrade Level {0} purchased for -{1} Money.", LevelAgility, NextLevelCost));
    }
    public void BuyAccelerationUpgrade()
    {// die prozentzahl pro stufe aus den maxstufen ausrechnen damit bei maxstufe doppelt so viel
        if (MaxUpgradeLevel == LevelAcceleration)
        {// maximale Zahl Stufen erreicht
            AddMessage(String.Format("You have already purchased {0} Acceleration Upgrades.", MaxUpgradeLevel));
            return;
        }
        int NextLevelCost = UpgradeCost(LevelAcceleration + 1);
        if (Money < NextLevelCost)
        {// zu wenig geld
            AddMessage(String.Format("You cannot afford Level {0} of Acceleration Upgrade (only {1} of {2} Money).", LevelAcceleration + 1, Money, NextLevelCost));
            return;
        }// alles ok, upgrade kaufen
        FactorAcceleration = 1.0f + (1.0f * (float)++LevelAcceleration / (float)MaxUpgradeLevel);    // beim maximalen upgrade level beschleunigt man 6 pixel pro sekunde
        Money -= NextLevelCost;
        AddMessage(String.Format("Acceleration Upgrade Level {0} purchased for -{1} Money.", LevelAcceleration, NextLevelCost));
    }
    public int BuyBrakeUpgrade()
    {// die prozentzahl pro stufe aus den maxstufen ausrechnen damit bei maxstufe voller stop
        if (MaxUpgradeLevel == LevelBrake)
        {// maximale Zahl Stufen erreicht
            AddMessage(String.Format("You have already purchased {0} Brake Upgrades.", MaxUpgradeLevel));
            return 0;
        }
        int NextLevelCost = UpgradeCost(LevelBrake + 1);
        if (Money < NextLevelCost)
        {// zu wenig geld
            AddMessage(String.Format("You cannot afford Level {0} of Brake Upgrade (only {1} of {2} Money).", LevelBrake + 1, Money, NextLevelCost));
            return 0;
        }// alles ok, upgrade kaufen
        FactorBrake = 1.0f - (1.0f * (float)++LevelBrake / (float)MaxUpgradeLevel);    // beim maximalen upgrade level bremst man total
        Money -= NextLevelCost;
        AddMessage(String.Format("Brake Upgrade Level {0} purchased for -{1} Money.", LevelBrake, NextLevelCost));
        return 1;
    }
    public float ActBrake()
    {// die aktuelle bremse zurückgeben
        if (ArtificIntelligence)    // die ki-spieler können voll bremsen ohne das upgrade zu kaufen
            return 0.0f;
        return BaseBrake * FactorBrake;
    }
    public float ActAcceleration()
    {// die aktuelle beschleunigung zurückgeben
        return BaseAcceleration * FactorAcceleration;
    }
    public float ActAgility()
    {// die aktuelle wendigkeit zurückgeben
        return BaseAgility * FactorAgility;
    }
    public float MaxFuel()
    {// die aktuelle maximale Menge Treibstoff zurückgeben
        return BaseFuel * FactorFuel;
    }
    public int MaxCargoCapacity()
    {// die aktuelle gesamte frachtkapazität zurückgeben
        return (int)((float)BaseCargoCapacity * FactorCargohold);
    }
    public int MaxHitpoints()
    {// die aktuelle panzerungsstärke zurückgeben
        return (int)((float)BaseHitpoints * FactorHitpoints);
    }
}

